% analyze model: run IRF and conditional variance decompositions
%
%

function analyzemodel(self)

  % get round options
  opt = self.get_round_options();

  % read the model
  load(self.fetch_file_name('read','model_mat',''));
  desc = cmop.expand_description(get(m, 'descript'), opt.rs_maturities);
  
  % run IRFs
  [sirfs,irfsrng] = srf(m, 1:opt.irfs_periods, 'size', @auto, ...
                    'select',opt.irfs_shocks);
  % run vardecomp
  vardecomp = fevd(m, opt.vardecomp_periods);
  
  % and save to the MAT file
  irfsenames = opt.irfs_shocks;
  save(self.fetch_file_name('write','analyze_model_mat'),...
       'sirfs', 'irfsenames', 'irfsrng', 'vardecomp');
  
  
  % report
  x = report.new(['Model dynamic properties report for round ' ...
                  self.name]);
  
  % add impulse response graphs
  for i = 1:length(opt.irfs_shocks)
    shk = opt.irfs_shocks{i};
    for j = 1:length(opt.irfs_vars)
      n = length(opt.irfs_vars{j});
      x.figure(['Response to the ' desc.(shk) ' (' shk ') -- page ' num2str(j) ],...
               'legend',false, 'zeroline', true, ...
               'plotoptions', {'LineWidth', 2});
      for k = 1:n
        var = opt.irfs_vars{j}{k};
        x.graph(desc.(var), 'range', [0:opt.irfs_periods]);
        x.series('', sirfs.(var){:,i});
      end
    end
  end
  
  % add variance decomposition graphs
  for i = 1:length(opt.vardecomp_variables)
    var = opt.vardecomp_variables{i};
    dsc = desc.(var);
    fh = vardecomp_process_one_variable(vardecomp, var, opt.vardecomp_contribs);
    %- add to the report
    x.userfigure(dsc, fh);
    x.pagebreak();
  end

  % add printout of the reduced form
  [T,R,K,Z,H,D] = sspace(m,'triangular',false);
  nb = size(T, 2);
  nf = size(T, 1) - nb;
  ne = size(R, 2);
  enames = get(m,'eList');
  xvec = get(m, 'xVector');
  %- find indexes of opt.vardecomp_variables in xvec
  ii = [];
  for i = 1:length(opt.vardecomp_variables)
    var = opt.vardecomp_variables{i};
    ii = [ii(:)', strmatch(var, xvec, 'exact')];
  end
  colNames = xvec(ii);
  %- set the matrix and rownames
  M = [R(ii,:), T(ii, :)]';
  rowNames = {enames{:}, xvec{nf+1:end}};
  %- do the matrix
  x.matrix(['Reduced form: columns are variables, rows are shocks and ' ...
            'states'],...
           M, 'long', true, 'rotateColNames', true,...
           'colNames', colNames, 'rowNames', rowNames, ...
           'longFoot', 'continued on next page');
  x.publish(self.fetch_file_name('write','analyze_model_pdf'),'display',false);
  
  
end

function fh = vardecomp_process_one_variable(x, var, ndecomp)
    
  shocks = colnames(x);
  
  %- find the variable in the x
  ii = strmatch(var, rownames(x), 'exact');
  if isempty(ii)
    error(['Could not find a variable ' var ' in decomposition']);
  end
    
  %- select ndecomp-1 largest contributions
  xx = double(squeeze(x(ii(1), :, :)))';
  xxsum = sum(xx, 1);
  [y, jj] = sort(xxsum, 2, 'descend');
  if size(xx, 2) > ndecomp-1
    yy = xx(:, jj(1:ndecomp-1));
    contribs = shocks(jj(1:ndecomp-1));
    %- add the rest as the last column to yy
    jjrest = setdiff([1:size(xx,2)],jj(1:ndecomp-1));
    yyrest = sum(xx(:,jjrest), 2);
    yy = [yy, yyrest];
    contribs{end+1} = 'REST';
  else
    yy = xx;
    contribs = shocks;
  end

  % make a figure
  fh = figure('visible','off');
  variance = sum(yy, 2);
  %- relative contributions
  subplot(2,1,1);
  yyrel = diag(variance)\yy*100;
  bar(yyrel,'stack');
  h = legend(contribs,'orientation','horizontal','location', ...
             'southoutside');
  set(h,'fontsize',8);
  set(h,'interpreter','none');
  grid;
  xlabel('time periods');
  ylabel('rel. contribution to variance (%)'); 
  xlim([0, size(yy, 1)+1]);
  ylim([0, 100]);
  subplot(2,1,2);
  plot([0:1:size(yy, 1)+1],[0;sqrt(variance);NaN],'LineWidth',2);
  grid;
  xlabel('time periods');
  ylabel('absolute standard error');
  xlim([0, size(yy, 1)+1]);
end
