classdef cmop
    
    properties (SetAccess = protected)
      % name of the forecasting round
      name = '';
      % root directory of the forecasting round
      dir = '';
      % prefix of the files which are expected to exist in the directory
      prefix = '';
    end
    
    properties (Constant = true)
      % name of the state file
      state_file = 'cmop_state.mat';
    end
    
    methods
      % constructor: construct from the existing directory
      % 
      % self = cmop(dir)
      %   construct from the given directory, name and prefix taken from
      %   the state file
      %
      % self = cmop(dir, name)
      %   construct from the given directory, prefix is taken from the
      %   state file, name is changed
      %      
      % self = cmop(dir, name, prefix)
      %   construct from the given directory, prefix and name taken from
      %   the user, state file is ignored
      function self = cmop(dir, varargin)
        if length(varargin) == 0 || length(varargin) == 1
          s = load([dir filesep cmop.state_file]);
        end
        
        self.dir = dir;
        if length(varargin) == 0
          self.name = s.name;
          self.prefix = s.prefix;
        elseif length(varargin) == 1
          self.name = varargin{1};
          self.prefix = s.prefix;
        elseif length(varargin) == 2
          self.name = varargin{1};
          self.prefix = varargin{2};
        else
          error('1, 2, or 3 arguments expected');
        end
        
        % save the state file
        warning('off','MATLAB:structOnObject');
        stmp = struct(self);
        warning('on','MATLAB:structOnObject');
        save([dir filesep cmop.state_file], '-struct', 'stmp');
      
        % make sure that results and reports directorie are there
        [tmp1, tmp2, tmp3] = mkdir([self.dir filesep 'results']);
        [tmp1, tmp2, tmp3] = mkdir([self.dir filesep 'reports']);
      end
      
      % copy contructor: creates an identical copy in the new directory
      %
      % newself = copy(self, newdir)
      %
      function newself = copy(self, newdir)
        % copy the directory
        if 7 ~= exist(newdir)
          error(['New directory ' newdir ' is not a directory']);
        end
        copyfile(self.dir, newdir);
        % create the new object
        newself = cmop(newdir, self.name, self.prefix);
      end

      % obtain round options: returns a struct of round options
      function opt = get_round_options(self, varargin)
        %- get options from the round options file
        f = self.fetch_file_name('run', 'round_options');
        [p,n,e] = fileparts(f);
        p = pathincr(p);
        clear(n);
        opt = eval([n '();']);
      
        %- override them from the command line
        optcmd = struct();
        ii = find(strcmp(varargin, '--'));
        if ~isempty(ii), optcmd = struct(varargin{ii(1)+1:end}); end
        opt = dbmerge(opt, optcmd);
        
        %- add default values for options
        if ~isfield(opt, 'use_sirius') % todo: document
          opt.use_sirius = false;
        end
        if ~isfield(opt, 'confidence_levels') % todo: document
          opt.confidence_levels = [0.9000 0.7000 0.5000 0.3000 0.1000];
        end
        if ~isfield(opt, 'history_report_range') % todo: document
          opt.history_report_range = opt.history_range;
        end
      end
      
      % read model and create the documentation
      readmodel(self)

      % analyze model: IRFs and variance decompositions
      analyzemodel(self)
    
      % observed data
      observeddata(self)
      
      % filter history
      filterhistory(self, scenario)
    
      % forecast
      forecast(self, scenario, varargin)
    
      % histforecast
      histforecast(self, rng)
    
      % do all round
      allround(self)
      
      % special methods
      special_report_uncondstderr(self, varargin)
      special_report_lossfunction(self, varargin)
      [pdflist,csvlist] = special_rolling_forecast(self, sc, srng, fend, shocks_ignore, varargin)
    end

    methods (Access=protected)
      % calculate/simulate confidence bands
      dcbands = confbands(self, m, d, dstd, anticipated_shocks, opt, sc)
    
      % create database of other frequency
      [do, fchar] = create_other_freq_database(self, d, sc, opt)
      
      % obtain the database of other frequency
      [do, fchar] = get_other_freq_database(self, sc, opt)
    end
    
    methods (Static)
      
      % simulate the model with surprise and anticipated information
      % combined
      [d, r] = combined_forecast(m, rng, din, endoexo_pairs, ...
                                 anticipated_shocks)

      % simulate the model with surprise and anticipated information
      % combined, compact version
      r = combined_forecast_compact(m, rng, din, surprise_pairs, ...
                                    anticipated_pairs, varargin)
      
      % process forecast report and add the tables and figures to the
      % report object
      x = process_forecast_report(x, m, dbsmean, dbsmean2, dbsstd, dcbands, labs, rd, opt)

      % process decomposition report and add the tables and figures to
      % the report object
      x = process_decomp_report(x, m, dc, labs, range, opt)
      
      % process historical forecast report and add graphs and tables to
      % the report object
      x = process_histforecast_report(x, m, r, opt)
      
      % decompose forecast change
      dc = decompose_forecast_change(c1, sc1, c2, sc2)

      % preprocess database using the FPAS mnemonics from user external
      % units to model internal units
      d = preprocess_database(d, ratesmat)

      % postprocess database using the FPAS mnemonics from model internal
      % units to user external units
      d = postprocess_database(d, ratesmat)

      % expand description defined by the model using the FPAS mnemonics
      % introducing the descriptions of the variables being introduced by
      % cmop.postprocess_database()
      desc = expand_description(desc, ratesmat)

      % expand islog information from the model using the FPAS mnemonics for the
      % variables being introduced by cmop.postprocess_database()
      desc = expand_islog(desc, ratesmat)

      % decompose the database results into the effect of grouped shocks
      % and initial condition
      [f fighs] = decomp_shocks(m, dbf, rng, varlist, groups, grpnames)
      
      % write xml file for scenario to be used by gui
      write_scenario_xml(outfile,sc,r,dtunes)
      
      % write xml file for report to be used by gui
      write_fcast_report_xml(outfile,rd,sc,opt,d,m)
      
    end
  end
