% simulate model with anticipated and surprise information
%
% r = combined_forecast_compact(m, rng, din, surprise_pairs, ...
%                                      anticipated_pairs)
%
% m                 model
% rng               forecasting range
% din               input database including sufficient initial condition
%                   before rng(1)
% surprise_pairs    struct mapping surprise shocks to endogenous
%                   variables which explain the shocks; if field value is
%                   empty, endogenous variable's tune is ignored and
%                   shock is taken directly from the database
% anticipated_pairs struct mapping antiicpated shocks to endogenous
%                   variables which explain the shocks (the same holds
%                   for empty matrices as for the case of surprise_pairs) 
% r                 structure of input, output database and plan for
%                   individual iterations; the iterations correspond to
%                   the surprises; the number of iterations can be lower
%                   than the total periods because at some point there
%                   does not need to be a new surprise information
%                   (non-zero shocks, or tuned endovariable paired with a
%                   surprise shock); the final resulting database is
%                   r.dbouts{end}
% varargin
%   linear          true/false, defaults to false

function r = combined_forecast_compact(m, rng, din, surprise_pairs, ...
                                       anticipated_pairs, varargin)
  
  args = struct(varargin{:});
  if ~isfield(args, 'linear')
    args.linear = false;
  end
  
  % initialize the resulting struct
  r = struct();
  r.dbins = {};
  r.dbouts = {};
  r.plans = {};

  d = din;

  % range of the input database
  dinrng = dbrange(din);

  % loop until there is no surprise information
  nswitch = 1;
  dt = rng(1);
  while nswitch > 0 && dt <= rng(end)
    if isa(m, 'sirius_dynamic_model')
      p = sirius_simulation_plan();
      p = p.create(m, dt:rng(end));
    else
      p = plan(m, dt:rng(end));
    end
    drunin = dbclip(d, dinrng(1):dt-1);
    
    % put surprise information to the dt period
    flds = fieldnames(surprise_pairs);
    for i = 1:length(flds)
      sexo = flds{i};
      sendo = surprise_pairs.(sexo);
      if ~isfield(drunin, sexo), drunin.(sexo) = tseries(); end
      if ~isempty(sendo) && ~isfield(drunin, sendo), drunin.(sendo) = tseries(); end
      if isfield(din, sendo) && ~isnan(din.(sendo)(dt))
        %- endo is tuned in this period
        if isa(p, 'sirius_simulation_plan')
          p.exogenize(sendo, dt);
          p.endogenize(sexo, dt);
        else
          p = exogenise(p, sendo, dt);
          p = endogenise(p, sexo, dt);
        end
        drunin.(sexo)(dt) = NaN;
        drunin.(sendo)(dt) = din.(sendo)(dt);
      else
        %- endo is not tuned in this period
        if ~isfield(din, sexo) || isnan(din.(sexo)(dt))
          drunin.(sexo)(dt) = 0;
        else
          drunin.(sexo)(dt) = din.(sexo)(dt);
        end
      end
    end
    
    % put anticipated information to the dt:rng(end)
    flds = fieldnames(anticipated_pairs);
    for i = 1:length(flds)
      aexo = flds{i};
      aendo = anticipated_pairs.(aexo);
      if ~isfield(drunin, aexo), drunin.(aexo) = tseries(); end
      if ~isempty(aendo) && ~isfield(drunin, aendo), drunin.(aendo) = tseries(); end
      rtune = []; % where aendo is tuned over dt:rng(end)
      % exogenize aendo where it is set in the din
      if ~isempty(aendo)
        if isfield(din, aendo)
          tmp = resize(din.(aendo), dt:rng(end));
          rtune = find(~isnan(tmp));
          if isa(p, 'sirius_simulation_plan')
            p.exogenize(aendo, rtune);
            p.endogenize(aexo, rtune);
          else
            p = exogenise(p, aendo, rtune);
            p = endogenise(p, aexo, rtune);
          end
          drunin.(aendo)(rtune) = din.(aendo)(rtune);
          drunin.(aexo)(rtune) = NaN;
        end
      end
      % put zeros to aexo where not specified and aendo not tuned
      rvoids = setdiff(dt:rng(end), rtune);
      rvoids = setdiff(rvoids, find(~isnan(din.(aexo))));
      drunin.(aexo)(rvoids) = 0;
      % put shocks to aexo where specified and aendo not tuned
      rxtune = setdiff(dt:rng(end), rtune);
      rxtune = setdiff(rxtune, find(isnan(din.(aexo))));
      drunin.(aexo)(rxtune) = din.(aexo)(rxtune);
    end
    
    % switch surprise information in din to zeros in drunin over
    % dt+1:rng(end)
    nswitch = 0;
    flds = fieldnames(surprise_pairs);
    for i = 1:length(flds)
      sexo = flds{i};
      sendo = surprise_pairs.(sexo);
      % set surprise endogenous to NaNs in the future
      if ~isempty(sendo)
        drunin.(sendo)(dt+1:rng(end)) = NaN;
      end
      % set surprise exogenous to zeros in the future
      drunin.(sexo)(dt+1:rng(end)) = 0.0;
      % calculate number of switches
      if isfield(din, sendo)
        xx = din.(sendo)(dt+1:rng(end));
        nnswitch = length(find(~isnan(xx)));
        nswitch = nswitch + nnswitch;
      end
      if isfield(din, sexo)
        xx = din.(sexo)(dt+1:rng(end));
        nnswitch = length(find(~isnan(xx) & xx ~= 0.0));
        nswitch = nswitch + nnswitch;
      end
    end
    
    % simulate
    if isa(m, 'sirius_dynamic_model') && ~args.linear
      %- sirius non-linear
      if length(r.dbouts) > 0
        d = m.solve(p, drunin, 'init', r.dbouts{end});
      else
        %- simulate only initial condition and use it as
        %- initialization in the first period
        pinit = sirius_simulation_plan();
        pinit.create(m, dt:rng(end));
        dinit = m.solve(pinit, dbclip(drunin, dinrng(1):dt-1)); 
        d = m.solve(p, drunin, 'init', dinit);
      end
    elseif isa(m, 'sirius_dynamic_model') && args.linear
      %- sirius linear
      d = m.lin_solve(p, drunin);
    else % iris
      if islinear(m) || args.linear, nnl = 0; else nnl = 20; end
      d = simulate(m, drunin, dt:rng(end), 'plan', p, 'anticipate', true, ...
                   'nonlinearize', nnl);
    end
    % save
    d = dboverlay(drunin, d);
    r.dbins{end+1} = drunin;
    r.dbouts{end+1} = d;
    if isa(p, 'sirius_simulation_plan')
      r.plans{end+1} = p.convert2db();
    else
      r.plans{end+1} = p;
    end
  
    % increment dt
    dt = dt+1;
  end
  
end

