function [f fighs] = decomp_shocks(m, dbf, rng, varlist, groups, grpnames)
%-historical shock decomposition
% Input arguments:
%   m         - parsed & solved model obj.
%   dbf       - input database with shocks
%   rng       - calculation range
%   varlist   - cell array of variables to decompose and report
%   groups    - cell array of cell arrays of group items
%   grpnames  - names for the groups
%
% Output:
%   f         - filtered dbase
%   fighs     -  handles to figures
%
% Michal Andrle, CNB-SMS, Oct. 2007, Prague
% Michal Andrle IMF/RES, Jan 2011 Jakarta, Indonesia
% Ondra Kamenik, Aug 2015
  
  %-get names
  elist = get(m,'elist');

  %-get endonames
  xlist = get(m,'xlist');

  %-get steady-state
  SS = get(m,'sstate');

  %- use 'iris' decomp for simplicity
  %- why deviations false??
  f  = simulate(m, dbf, rng, 'anticipate', false, 'deviations', false, 'contributions', true);

  %-get labels
  %'fnames = get(f.(xlist{1}),'comment');
  enames = get(m,'elist');    
  fnames = cell(numel(enames)+1,1);
  fnames{end} = 'Initial conditions';
  fnames(1:numel(enames)) = enames;

  %-use grouping & then report

  %--span the space of shocks
  [groups grpnames] = add_rest_(groups, grpnames, fnames);

  %--storage
  contribmat = zeros(length(rng),length(groups),length(varlist));
  fighs = []; % handles to graphs

  for i = 1:length(varlist)
    %-create groupings
    for g = 1:length(groups)
      for n = 1:length(groups{g})
        ixf = strmatch(groups{g}{n},fnames,'exact');
        if isempty(ixf)
          fprintf('Cannot find shock: %s\n',groups{g}{n});
        end
        %sf  = dbfetch(f,ixf);
        sf  = dbcol(f,ixf);   
        %--assumes DEVIATIONS from a SState
        contribmat(:,g,i) = contribmat(:,g,i) + (sf.(varlist{i})(rng));       
      end
    end 
  end


  for i = 1:length(varlist)
    [fh] = conbarplot_(contribmat(:,:,i),rng,varlist{i},grpnames);
    fighs{end+1} = fh;
  end


end %-of the main function



function [groups grpnames] = add_rest_(groups, grpnames, fnames)
%-- creates a new group/groupname if needed for 'rest'
  is_registered = zeros(length(fnames),1);
  for j = 1:length(groups)
    for k = 1:length(groups{j})
      ix = strmatch(groups{j}{k},fnames,'exact');     
      is_registered(ix) = 1;  
    end
  end

  if any(is_registered==0)
    iix = find(is_registered==0);
    groups{end+1} = fnames(iix);
    grpnames{end+1} = '-Rest-';
  end

end %-of the subfunction


function [fh] = conbarplot_(u, rng, varname, groupnames)
%--plot the bar chart
  %-split
  upos=zeros(size(u));
  upos(find(u>0)) = u(find(u>0));
  uneg=zeros(size(u));
  uneg(find(u<0)) = u(find(u<0));
  %-tserize
  tspos = tseries(rng,upos);
  tsneg = tseries(rng,uneg);

  %-figure
  fh = figure('visible','off','Position', [500 200 450*2 250*2]);
  bar(tspos,'stack');hold on;
  bar(tsneg,'stack');hold on;

  %-plot sum
  cmsm = sum(tspos+tsneg,2);
  plot(rng,cmsm,'linewidth',4, 'color', [1 1 1 ]);
  pp = plot(rng,cmsm,'linewidth',3, 'color', 'k');
  grid;
  
  axis_y_limits = get(gca, 'YLim');
  set(gca, 'YLim', axis_y_limits.*[1 1.35])
  
  %-put legend
  ll = legend(groupnames,'location','NorthWest');
  
   axis_y_limits = get(gca, 'YLim');
  set(gca, 'YLim', axis_y_limits.*[1 1])
  
  %-title
  title(varname,'interpreter','none');
end



