% runs the decomposition of the differences between two forecasts into:
% model change, surprise shocks information, initial state, and
% individual shocks
%
% dc = decompose_forecast_change(c1, sc1, c2, sc2)
%
%   decompose forecast change [c1,sc1] minus [c2,sc2]
%
% c1    cmop object for the first forecast
% sc1   scenario name of the first forecast within c1
% c2    cmop object for the second forecast
% sc2   scenario name of the second forecast within c2
% dc    structure with the contributions explaining [c1,sc1] minus [c2,sc2]

function dc = decompose_forecast_change(c1, sc1, c2, sc2)
  
  % options of the c1
  opt = c1.get_round_options();
  
  % get information from c1 and c2
  %- models
  %-- first model
  [fn, isthere] = c1.fetch_file_name('read','model_mat',['_' sc1]);
  if isthere
    load(fn);
  else
    load(c1.fetch_file_name('read','model_mat',''));
  end
  m1 = m;
  mm1 = m;
  if opt.use_sirius
    tmp = sirmexmod('iris_model', m1);
    mm1 = tmp.sdm_handle;
  end
  %-- second model
  [fn, isthere] = c2.fetch_file_name('read','model_mat',['_' sc2]);
  if isthere
    load(fn);
  else
    load(c2.fetch_file_name('read','model_mat',''));
  end
  m2 = m;
  %- get forecast results
  load(c1.fetch_file_name('read','scenario_forecast_mat',sc1));
  r1 = r;
  dout1 = r1.dbouts{end};
  load(c2.fetch_file_name('read','scenario_forecast_mat',sc2));
  r2 = r;
  dout2 = r2.dbouts{end};

  % decomposition: will start with [c2,sc2] and gradually get to [c1,sc1]

  % collect elements of the decomposition: models [m1, m2], surprise info
  % [s1,s2], init [i1, i2], and shocks data [d1,d2]
  rng = r1.range;
  s1 = fieldnames(r1.anticipated_pairs);
  s2 = fieldnames(r2.anticipated_pairs);
  i1 = get_init_db(m1, dout1, rng(1));
  i2 = get_init_db(m2, dout2, rng(1));
  d1 = get_shocks_db(m1, dout1, rng);
  d2 = get_shocks_db(m2, dout2, rng);

  % get adjusted information: s2 for model m1, i2 for model m1, d2 for
  % model m1
  s2a = adjust_anticipated_for_model(m1, s2);
  i2a = adjust_init_for_model(m1, i2);
  d2a = adjust_shocks_for_model(m1, d2);

  dc = struct();
  dc.dout1 = dout1;
  dc.dout2 = dout2;
  % difference model: [m1,s2a,i2a,d2a] minus [m2,s2,i2,d2] 
  disp('  impact of model change');
  fm = simulate_shocks(mm1, s2a, i2a, d2a);
  dc.model = dbminuscontrol(m1, fm, dout2);
  % difference surprise: [m1,s1,i2a,d2a] minus [m1,s2a,i2a,d2a] 
  disp('  impact of surprise/anticipated change');
  fs = simulate_shocks(mm1, s1, i2a, d2a);
  dc.surprise = dbminuscontrol(m1, fs, fm);
  % difference init: [m1,s1,i1,d2a] minus [m1,s1,i2a,d2a]
  disp('  impact of initial condition change');
  fi = simulate_shocks(mm1, s1, i1, d2a);
  dc.init = dbminuscontrol(m1, fi, fs);
  % difference shocks: [m1,s1,i1,d1] minus [m1,s1,i1,d2a]
  disp('  impact of shocks change');
  dc = decompose_shocks(dc, m1, mm1, s1, i1, d2a, d1, fi);
  % calculate effects of non-linearity:
  % dout1-dout2-(model+surprise+init+total_shocks)
  disp('  impact of model non-linearity');
  logs = get(m1, 'log');
  flds = fieldnames(logs);
  logvars = flds(cell2mat(struct2cell(logs))==1);
  nonlogvars = flds(cell2mat(struct2cell(logs))==0);
  logvars = intersect(logvars, get(m2, 'xlist'));
  nonlogvars = intersect(nonlogvars, get(m2, 'xlist'));
  deltan = dbbatch(dout1, '$0',...
                   'dout1.$0-(dout2.$0+dc.model.$0+dc.surprise.$0+dc.init.$0+dc.total_shocks.$0)',...
                   'namelist',nonlogvars,'fresh',true);
  deltal = dbbatch(dout1, '$0',...
                   'dout1.$0/(dout2.$0*dc.model.$0*dc.surprise.$0*dc.init.$0*dc.total_shocks.$0)',...
                   'namelist',logvars,'fresh',true);
  dc.nonlinear = deltan + deltal;
end


% this fills the database dbout with initial condition for the given model m
% taking the data from dbin considering that first forecasting period is dt
function dbout = get_init_db(m, dbin, dt)
  
  dbout = struct();
  initcond = get(m, 'initcond');
  for i = 1:length(initcond)
    % parse initcond{i}
    g = regexp(initcond{i}, '(?<var>\w+)\{-(?<lag>[0-9]+)\}', 'names');
    % set the variables in the database
    if ~isfield(dbout, g.var)
      dbout.(g.var) = tseries();
    end
    dbout.(g.var)(dt-str2num(g.lag)) = dbin.(g.var)(dt-str2num(g.lag));
  end
end

% this fills the database dbout with shocks from the given database dbin
% over the range rng. all NaN shocks are turned to zeros
function dbout = get_shocks_db(m, dbin, rng)
  
  dbout = struct();
  enames = get(m, 'elist');

  for i=1:length(enames)
    e = enames{i};
    ts = dbin.(e)(rng);
    ts(isnan(ts)) = 0;
    dbout.(e) = tseries(rng, ts);
  end
end

% we just take intersection of all shocks in the new model and given
% anticipated shocks
function sa = adjust_anticipated_for_model(m, s)
  sa = intersect(get(m,'enames'), s);
end

% for all variable which are not in the initi condition, we apply the
% steady state of the new model
function ia = adjust_init_for_model(m, i)
  rng = dbrange(i);
  mlag = get(m,'maxlag');
  ssd = sstatedb(m, rng(end)+mlag : rng(end));
  inew = get_init_db(m, ssd, rng(end)+1);
  ia = dboverlay(inew, i);
end

% we add zero shocks to the database which are not in the d, and drop
% shocks from the database which are 
function da = adjust_shocks_for_model(m, d)
  rng = dbrange(d);
  da = d;
  todrop = setdiff(fieldnames(d), get(m,'enames'));
  toadd = setdiff(get(m,'enames'), fieldnames(d));

  da = rmfield(da, todrop);
  for i = 1:length(toadd)
    da.(toadd{i}) = tseries(rng, 0);
  end
end

% simulate the shocks with anticipated/surprise status
%
% f = simulate_shocks(m, s, i, d)
%
% m      model or sirius_dynamic_model
% s      surprise/anticipated information: this is a cell array of
%        anticipated shocks
% i      init condition, database
% d      database of the shocks, gives the range of the forecast

function f = simulate_shocks(m, s, i, d, varargin)
  
  rng = dbrange(d);
  dbin = i + d;
  anticipated_list = s;
  if isa(m, 'sirius_dynamic_model')
    enames = m.exo_names();
  else
    enames = get(m, 'enames');
  end
  surprise_list = setdiff(enames, anticipated_list);
  surprise_pairs = cell2struct(cell(1,length(surprise_list)), surprise_list,2);
  anticipated_pairs = cell2struct(cell(1,length(anticipated_list)), anticipated_list,2);
  r = cmop.combined_forecast_compact(m, rng, dbin, surprise_pairs, ...
                                     anticipated_pairs, varargin{:});
  f = r.dbouts{end};
end



% this function decomposes impact of change of shocks database from d2 to
% d1 into individual shocks, where d2out is an output database
% corresponding to d2
function dc = decompose_shocks(dc, m, mm, s, init, d2, d1, d2out);

  enames = get(m, 'enames');
  % decompose to the individual shocks, by accummulating to drun
  dc.individual_shocks = struct();
  dblast = d2out;
  drun = d2;
  for i = 1:length(enames)
    e = enames{i};
    drun.(e) = d1.(e);
    if norm(d2.(e)-d1.(e)) > 1e-11
      disp(['    impact of shock ' e ' change']);
      f = simulate_shocks(mm, s, init, drun, 'linear', false); 
      %todo: switch for linear=true
    else
      f = dblast;
    end
    dc.individual_shocks.(e) = dbminuscontrol(m, f, dblast);
    dblast = f;
  end
  
  % write total difference caused by the shocks
  dc.total_shocks = dbminuscontrol(m, dblast, d2out);
end
