% run the forecast for the given scenario
%
% forecast(self, sc)
% forecast(self, sc, sc2)
% forecast(self, sc, self2, sc2)
%

function forecast(self, sc, varargin)
  % get round options
  opt = self.get_round_options(varargin{:});
  ii = find(strcmp(varargin, '--'));
  if ~isempty(ii), varargin = varargin(1:ii(1)-1); end
  
  % check that the sc is one of the scenarios
  if isempty(strmatch(sc, opt.round_scenarios, 'exact'))
    error(['Scenario ' sc ' is not among the round scenarios']);
  end
  
  % read the model
  [fn, isthere] = self.fetch_file_name('read','model_mat',['_' sc]);
  if isthere
    load(fn);
  else
    load(self.fetch_file_name('read','model_mat',''));
  end

  % make sirius model if needed
  mm = m;
  if (isfield(opt,'use_sirius') && opt.use_sirius)
    tmp = sirmexmod('iris_model',m,'omit_measurements',true);
    mm = tmp.sdm_handle;
  end
  
  % get user tunes
  dtunes = dbload(self.fetch_file_name('read','scenario_tunes', sc));  
  dtunes = cmop.preprocess_database(dtunes, opt.rs_maturities);
  % get filtered database
  if isfield(opt, 'hist_use') && isfield(opt.hist_use, sc)
    sch = opt.hist_use.(sc);
  else
    sch = sc;
  end
  dhist = dbload(self.fetch_file_name('read', 'scenario_history_mean', sch));
  dhist = dbclip(dhist, opt.history_range(1):opt.forecast_range(1)-1);
    
  % get definition of the scenario
  f = self.fetch_file_name('run','define_scenario', sc);
  [p,n,e] = fileparts(f);
  p = pathincr(p);
  clear(n);
  sd = eval([n '();']);

  % simulate the forecast
  din = dboverlay(dhist, dtunes);
  %- overlay it with steady state database for lags
  dlags = sstatedb(m, opt.forecast_range(1));
  dlags = dbclip(dlags, opt.forecast_range(1)-1+get(m,'maxlag'):opt.forecast_range(1)-1);
  din = dboverlay(dlags, din);
  %- combined forecast
  [d, r] = cmop.combined_forecast(mm, opt.forecast_range, din, ...
                                  sd.endo_tunes_by_exo, ...
                                  sd.anticipated_shocks);
  d = cmop.postprocess_database(d, opt.rs_maturities);
  [do, fchar] = self.create_other_freq_database(d, sc, opt);
  
  % save the results
  dbsave(d, self.fetch_file_name('write','scenario_forecast_csv', sc));
  save(self.fetch_file_name('write','scenario_forecast_mat', sc),...
       'r');
  if ~isempty(fchar)
    dbsave(do, self.fetch_file_name('write', 'scenario_forecast_csv_otherfreq', ...
                                    sc, fchar),...
           'dateformat','YYYYFP');
  end
  
%   % write the scenario xml for gui
%   cmop.write_scenario_xml(self.fetch_file_name('write','scenario_xml',sc),sc,r,dtunes);

  % run iris kalman for whole range without tunes to get the stds
  dobs = dbload(self.fetch_file_name('read','observed_data'));
  dobs = cmop.preprocess_database(dobs, opt.rs_maturities);
  [~,out] = filter(m, dobs, opt.history_range(1):opt.forecast_range(end),'data','smooth',...
                   'relative',false);
  dstd = out.std;
  %- save
  dbsave(dstd, self.fetch_file_name('write','scenario_forecast_std',sc));

  % confidence bands
  dcbands = confbands(self, m, d, dstd, sd.anticipated_shocks, opt, sc);
  
  % report the scenario
  f = self.fetch_file_name('run','define_forecast_report');
  [p,n,e] = fileparts(f);
  p = pathincr(p);
  clear(n);
  if ~isstruct(opt.model_linear), swt = opt.model_switches; ...
  else swt = opt.model_switches.(sc); end
  rd = eval([n '(swt);']);
  if length(varargin) == 0
    % single scenario, no comparison
    reptitle = ['Forecast report round: ' self.name ', scenario: ' sc];
    dbs = {d};
    dbso = {do};
    names = {[self.name '-' sc]};
    filename = self.fetch_file_name('write','scenario_forecast_pdf', sc);    
  else
    % compare the scenario sc with (self2/)sc2
    if length(varargin) > 1
      self2 = varargin{1};
      sc2 = varargin{2};
    else
      self2 = self;
      sc2 = varargin{1};
    end
    reptitle = ['Forecast report: ' self.name '[' sc ']' ...
                ' vrs. ' self2.name '[' sc2 ']' ];
    d2 = dbload(self2.fetch_file_name('read','scenario_forecast_csv', sc2));
    dbs = {d, d2};
    do2 = self2.get_other_freq_database(sc2, opt);
    dbso = {do, do2};
    names = {[self.name '-' sc], [self2.name '-' sc2]};
    filename = self.fetch_file_name('write','scenario_forecast_pdf', sc, ...
                                    self2.name, sc2);
  end
  x = report.new(reptitle);
  warning('off','MATLAB:graphicsversion:GraphicsVersionRemoval');
  x = cmop.process_forecast_report(x, m, dbs, dbso, {dstd}, dcbands, names, rd, opt);
  x.publish(filename, 'display',false,'package',{'relsize'});
  warning('on','MATLAB:graphicsversion:GraphicsVersionRemoval');
    
%   % write the report definition xml for gui
%   cmop.write_fcast_report_xml(self.fetch_file_name('write','fcast_report_xml',sc),...
%                               rd,sc,opt,d,m);  
%   
  % decompose the scenario changes
%   if length(varargin) == 1
%     if length(varargin) > 1
%       self2 = varargin{1};
%       sc2 = varargin{2};
%     else
%       self2 = self;
%       sc2 = varargin{1};
%     end
%     decomp = cmop.decompose_forecast_change(self, sc, self2, sc2);
%     % save to the mat file
%     save(self.fetch_file_name('write','scenario_change_decomp_mat',sc,self2.name,sc2),...
%          'decomp');
%     % generate report
%     x = report.new(['Forecast change decomposition: ' names{1} ' vrs. ' names{2}]);
%     x = cmop.process_decomp_report(x, m, decomp, names, r.range, opt);
%     x.publish(self.fetch_file_name('write','scenario_change_decomp_pdf',sc,self2.name,sc2),...
%               'display',false);
%   end
end
