% create decomposition report
%
% x         report object
% m         model object
% dc        decomp structure as output from the method
%           cmop.decompose_forecast_change
% labs      cell of 2 strings of the names corresponding to scenarios
% rng       range for the graphs and tables

function  x = process_decomp_report(x, m, dc, labs, rng, opt)

  % shock labels
  desc = cmop.expand_description(get(m, 'descript'), opt.rs_maturities);
  enames = get(m, 'elist');
  sh_labels = {};
  for i = 1:length(enames)
    pdsc = regexp(desc.(enames{i}), '(?<com>.*) +\((?<unit>.*)\) *', 'names');
    if length(pdsc) > 0
      sh_labels{end+1} = pdsc(1).com;
    else
      sh_labels{end+1} = desc.(enames{i});
    end
  end
  
  % decomposition labels
  dc_labels = {'Model','Surprise','Init','Nonlinear',...
               sh_labels{:}};
  
  % logvar information
  islog = get(m, 'log');
  
  for i = 1:length(opt.decomp_variables)
    var = opt.decomp_variables{i};
    % form the time series of the decomposition: rows periods, columns dc_labels
    a = tseries(rng, zeros(length(rng), length(dc_labels)));
    %- write changes from the first four parts
    for j = 1:4
      dlab = lower(dc_labels{j});
      a(rng, j) = dc.(dlab).(var)(rng);
    end
    %- write changes of the shocks
    for j = 5:length(dc_labels)
      shockname = enames{j-4};
      a(rng, j) = dc.individual_shocks.(shockname).(var)(rng);
    end
    %- make it 100*log if it is a log variable
    if islog.(var), a = 100*log(a); end
    
    % select ncontrib-1 largest contributions and put rest to the Rest
    [asel, sel_labels] = select_largest_contribs(a, rng, dc_labels, opt.decomp_contribs);
    
    % draw the two graphs (upper the variables, lower the decomposition)
    fh = figure('visible','off');
    %- upper plot
    yy = [dc.dout1.(var),dc.dout2.(var)];
    if islog.(var), yy = 100*log(yy); end
    subplot(2,1,1);
    plot(rng, yy, 'LineWidth', 2);
    grid;
    axis tight;
    legend(labs, 'location','SouthOutside','orientation','horizontal');
    %- lower plot
    subplot(2,1,2);
    bar_plot(asel, rng);
    grid;
    axis tight;
    legend(sel_labels, 'location','SouthOutside','orientation','horizontal');
    %- userfigure
    tit = ['Decomposition of forecast changes: ' desc.(var)];
    if islog.(var), tit = [tit ' (trans. to 100*log)']; end
    x.userfigure(tit, fh);
    
    % make the detailed table
    x.table(tit, 'range', rng, 'dateformat','YYFP');
    x.series(labs{1}, yy{:,1});
    x.series(labs{2}, yy{:,2});
    x.series('Difference', yy{:,1}-yy{:,2}, 'separator','\hline');
    for j = 1:4
      dlab = lower(dc_labels{j});
      x.series(dc_labels{j}, dc.(dlab).(var));
    end
    x.series('Total shocks', dc.total_shocks.(var), 'separator','\hline');
    x.series('Individual shocks', tseries(),'nan',' ');
    [xsel, xlabs] = select_largest_contribs(a{:,5:end}, rng, sh_labels, ...
                                            2*opt.decomp_contribs);
    for j = 1:length(xlabs)
      x.series(xlabs{j}, xsel{:,j}); 
    end
  end
end

function bar_plot(u, prng)
  
  % divide to positive and negative
  u = u(prng);
  upos = u;
  upos(find(u < 0)) = 0;
  uneg = u;
  uneg(find(u > 0)) = 0;
  upos = tseries(prng, upos);
  uneg = tseries(prng, uneg);
  % plot it
  bar(prng, upos,'stack'); hold on;
  bar(prng, uneg,'stack'); hold on;
end

function [asel, sellabs] = select_largest_contribs(a, rng, labs, ncontrib)
  nrm2 = sum(a(rng).^2,1).^(1/2);
  [srnm2,ii] = sort(nrm2, 'descend');
  asel = tseries(rng, zeros(length(rng), ncontrib));
  asel(:, 1:ncontrib-1) = a(:,ii(1:ncontrib-1));
  asel(:, ncontrib) = sum(a(:,ii(ncontrib:end)),2);
  sellabs = labs(ii(1:ncontrib-1));
  sellabs{end+1} = 'Rest';
end
