function x = process_histforecast_report(x, m, r, opt)
  
  % variable descriptions
  desc = cmop.expand_description(get(m, 'descript'), opt.rs_maturities);
  % steady state and stationary variables
  ss = get(m, 'sstate');
  stat = get(m, 'stationary');
  
  % get the latest observations
  rend = r{end};
  dobs = dbclip(rend.dbsmooth, rend.hrng(1):rend.dtm);
  
  % graphs
  for i = 1:length(opt.histfcast_vars)
    var = opt.histfcast_vars{i};
    ssvar = NaN;
    if stat.(var), ssvar = ss.(var); end
    show_var = '';
    if isfield(opt.histfcast_show_variable, var)
      show_var = opt.histfcast_show_variable.(var);
    end
    x = add_histfcast_graph(x, var, desc.(var), r, dobs, ssvar, ...
                            opt.histfcast_range_left, show_var, opt.histfcast_plot_hist);
    x.pagebreak();
  end
  
  % rmse
  x = add_histfcast_rmse(x, m, opt.histfcast_vars, r, dobs, opt.histfcast_nahead);
  
end

function x = add_histfcast_graph(x, var, vardesc, r, dobs, ssvar, range_left, ...
                                 show_var, plot_hist)
  
  % determine the range of whole the graph
  prng = r{1}.frng(1)-range_left : r{end}.frng(end);

  % determine the frequency of the forecasts and model
  ffcast = datfreq(r{1}.dt);
  fmodel = datfreq(r{1}.frng(1));
  np = ceil(ffcast/fmodel);
  legflag = zeros(1,np); 
  freqletters = {'Y','H','','Q','','','','','','','','M'};
  lfcast = freqletters{ffcast};
  lmodel = freqletters{fmodel};
  
  % choose np colors different from black
  colors = zeros(np,3);
  colors(1:4:end) = 1;
  
  % figure
  x.figure(['Historical simulations for ' vardesc],...
           'range', prng);
  if any(legflag)
      x.graph('','legend',true,'legendlocation','southoutside',...
          'postprocess', 'set(L,''orientation'',''horizontal'');');
  else
      x.graph('','legend',true,'legendlocation','southoutside');
  end


  %- final variable outcome
  x.series('', dobs.(var), 'plotOptions',{'LineWidth',2,'color','black'},...
           'legendEntry',NaN);
    
  if ~isempty(show_var)
    x.series('', dobs.(show_var), 'plotOptions',{'LineWidth',1,'color','black'},...
             'legendEntry',NaN);
  end
    
  %- historical forecasts
  for i = 1:length(r)
    %- range of the forecast line: include last historical period
    hfrng = r{i}.frng(1)-1 : r{i}.frng(end);
    %- find out position of r{i}.dt within r{i}.frng(1)
    tmp = convert(r{i}.frng(1), ffcast);
    pos = rem(abs(tmp - r{i}.dt), np)+1;
    %- draw the graph
    popt = {'color', colors(pos,:), 'Linestyle', '--'};
    x.series('',resize(r{i}.dbfcast.(var), hfrng),...
             'plotOptions',popt,'legendEntry',NaN);
    extras = {'legendEntry',NaN};
    if ~legflag(pos) && np > 1
      extras = {'legendEntry',[num2str(pos) lfcast ' in ' lmodel]};
      legflag(pos) = 1;
    end
    popt = {popt{:}, 'Marker','*'};
    x.series('',resize(r{i}.dbfcast.(var), hfrng(1)), 'plotOptions',popt,extras{:});
    if plot_hist > 0
      x.series('',resize(r{i}.dbfcast.(var), hfrng(1)-plot_hist:hfrng(1)),...
               'plotOptions',{'color','black','LineWidth',1},...
               'legendEntry',NaN);
    end
  end
  %- steady state line
  if ~isnan(ssvar)
    x.series('', tseries(prng, ssvar), ...
             'plotOptions',{'Color','black','LineWidth',1.5,'Linestyle','--'},...
             'legendEntry',NaN);
  end
end

function x = add_histfcast_rmse(x, m, vars, r, dobs, nahead)

  rngobs = dbrange(dobs);
  err2 = cell(1,length(vars));
  nums = cell(1,length(vars));
  
  % accumulate squares of errors and number of hits
  for i = 1:length(vars)
    var = vars{i};
    err2{i} = zeros(length(rngobs), length(nahead));
    nums{i} = zeros(1,length(nahead));
    for j = 1:length(r)
      rrng = r{j}.frng(1)-1+nahead; % range of the nahead forecasts
      rrng = intersect(rrng, rngobs); % make sure that the range does not
                                      % go beyond the end
      nnahead = nahead(1:length(rrng)); % the same for nahead
      iir = rrng-rngobs(1)+1; % row indicies in the matrix err2
      iilinr = sub2ind(size(err2{i}), iir, 1:length(nnahead)); % linear index in err2{i}
      err2{i}(iilinr) = err2{i}(iilinr) + (dobs.(var)(rrng)' - r{j}.dbfcast.(var)(rrng)').^2;
      nums{i}(1:length(nnahead)) = nums{i}(1:length(nnahead)) + 1;
    end
  end
  
  % now calculate the RMSE tables
  rmse = cell(1, length(vars));
  for i = 1:length(vars)
    rmse{i} = (sum(err2{i},1)./nums{i}).^(1/2);
  end
  
  % now make the table
  desc = get(m,'descript');
  heading = {'Variable'};
  for i = 1:length(nahead)
    heading{end+1} = num2str(nahead(i));
  end
  A = cell(length(vars),length(nahead)+1);
  for i = 1:length(vars)
    var = vars{i};
    A{i,1} = desc.(var);
    for j = 1:length(nahead);
      A{i,j+1} = rmse{i}(j);
    end
  end
  
  % add the array
  x.array(['RMSE for the range ' dat2char(r{1}.frng(1)-1+nahead(1)) ...
           ' -- ' dat2char(rngobs(end))], A,...
          'heading', heading);
end

