% Special ad-hoc report for Doug Laxton displaying:
% 1. sample mean
% 2. sample stderr
% 3. theoretical unconditional stderr for model with IFB rule
% 4. same as 3 but with SHK_RS shut-off
% 5. same as 4 but for optimal policy model
% for variables: change of RS, core YoY, headline YoY and output gap
%
% optional parameters
% shut_off_shocks     cell array of shocks to be shut-off in all models
% policy_shock        name of the policy shock (defaults to SHK_RS)
% add_calibrations    cell array of structs setting alternative
%                     calibrations

function special_report_uncondstderr(self, varargin)
  
  opthis = struct(varargin{:});
  if ~isfield(opthis, 'shut_off_shocks')
    opthis.shut_off_shocks = {};
  end
  if ~isfield(opthis, 'policy_shock')
    opthis.policy_shock = 'SHK_RS';
  end
  if ~isfield(opthis, 'add_calibrations')
    opthis.add_calibrations = {};
  end
  
  % set up a few things
  bsmodel = 'baseline';
  optmodel = 'optbaseline';
  vars = {'DRS','D4L_CPIXFE','D4L_CPI','L_GDP_GAP'};

  % get round options
  opt = self.get_round_options();
  hrng = opt.history_range(1):opt.forecast_range(1)-1;
  
  % results matrix
  calibs = {struct(), opthis.add_calibrations{:}};
  resA = zeros(length(vars), 2+3*length(calibs));
  
  % get sample means and stderrs
  dhist = dbload(self.fetch_file_name('read', 'scenario_history_mean', bsmodel));
  for i = 1:length(vars)
    resA(i,1) = mean(dhist.(vars{i})(hrng));
    resA(i,2) = std(dhist.(vars{i})(hrng));
  end
  
  % load models
  a1 = load(self.fetch_file_name('read','model_mat',['_' bsmodel]));
  a2 = load(self.fetch_file_name('read','model_mat',['_' optmodel]));
  mm = {a1.m, a1.m, a2.m};
  for i = 1:length(mm)
    for j = 1:length(opthis.shut_off_shocks)
      mm{i}.(['std_' opthis.shut_off_shocks]) = 0;
    end
  end
  %- kill policy shock in the latter two
  mm{2}.(['std_' opthis.policy_shock]) = 0;
  mm{3}.(['std_' opthis.policy_shock]) = 0;
  
  % calculate unconditional stderrs for the models
  for c = 1:length(calibs)
    for i = 1:length(mm)
      %- solve the model
      m = assign(mm{i}, calibs{c});
      m = solve(m);
      %- calculate theoretical unconditional errors for the three models
      C = acf(m,'select',vars);
      C  = sqrt(diag(C));
      resA(:, 2+3*(c-1)+i) = C;
    end
  end
  
  % report the table
  rngstr = [dat2char(hrng(1)) '---' dat2char(hrng(end))];
  A = cell(length(vars), 1+size(resA,2));
  desc = get(mm{1},'comments');
  for i = 1:length(vars)
    A{i,1} = desc.(vars{i});
  end
  A(:,2:end) = num2cell(resA);
  %- actual report
  x = report.new(['Special report on unconditional std. errors for round ' ...
                  self.name]);
  leg = {'A',['Sample mean over ' rngstr];...
         'B',['Sample stderr over ' rngstr];...
         'C','Theoretical stderr for baseline';...
         'D','Theoretical stderr for baseline without policy shock';...
         'E','Theoretical stderr for optimal policy without policy shock'};
  for c=1:length(opthis.add_calibrations)
    flds = fieldnames(opthis.add_calibrations{c});
    str = '';
    for i = 1:length(flds)
      str = [str flds{i} '=' num2str(opthis.add_calibrations{c}.(flds{i})) ...
             ' '];
    end
    leg{end+1,1} = char(69+(c-1)*3+1);
    leg{end,2} = ['Same as C with ' str];
    leg{end+1,1} = char(69+(c-1)*3+2);
    leg{end,2} = ['Same as D with ' str];
    leg{end+1,1} = char(69+(c-1)*3+3);
    leg{end,2} = ['Same as E with ' str];
  end
  
  x.array('Legend to the following table',...
          leg,...
          'heading',{'Column','Description'});
  head = num2cell(char(65:65+2+length(calibs)*3-1));
  head = {'', head{:}};
  x.array('Unconditional sample and theoretical std. errors',...
          A,...
          'heading',head);
  x.publish(self.fetch_file_name('write','special_report_uncondstderr'),...
            'display',false);
end
