% Run the rolling forecasts from the histprical shocks. All shocks come
% as surprise.
% 
% pdflist       list of PDF files from this action
% csvlist       list of CSV database files from this action
% sc            scenario name (model will be used and historical
%               interpretation (unless hist_use says otherwise)
% srng          Iris range for the starting periods
% fend          Iris date for the last period of each forecast
% shocks_ignore cell array of the shocks which should not be taken for
%               the future
% varargin:
%    hs         cell: rows by 2 columns, each i-th row sets homotopy
%               steps hs{i,2} simulation starting in hs{i,1} 
%    ns         number of future shocks taken in each period (defaults to 1)
%    compareto  if 'prev', compares each iteration to the previous one,
%               if 'hist', compares to history (default is 'prev')

function [pdflist,csvlist] = special_rolling_forecast(self, sc, srng, fend, shocks_ignore, varargin)
  
  % process arguments
  args = struct(varargin{:});
  %- homotopy steps for each of the dates
  hss = repmat(1, 1, length(srng));
  if isfield(args, 'hs')
    for i=1:size(args.hs,1)
      jj = find(args.hs{i,1} == srng);
      if ~isempty(jj)
        hss(jj) = args.hs{i,2};
      end
    end
  end
  %- number of shocks to take defaults to 1
  if ~isfield(args, 'ns')
    args.ns = 1;
  end
  %- compareto defaults to prev
  if ~isfield(args, 'compareto')
    args.compareto = 'prev';
  end
  
  % get round options
  opt = self.get_round_options();

  % check that the sc is one of the scenarios
  if isempty(strmatch(sc, opt.round_scenarios, 'exact'))
    error(['Scenario ' sc ' is not among the round scenarios']);
  end
  
  % read the model
  [fn, isthere] = self.fetch_file_name('read','model_mat',['_' sc]);
  if isthere
    load(fn);
  else
    load(self.fetch_file_name('read','model_mat',''));
  end

  % make sirius model if needed
  mm = m;
  if (isfield(opt,'use_sirius') && opt.use_sirius)
    tmp = sirmexmod('iris_model',m,'omit_measurements',true);
    mm = tmp.sdm_handle;
  end
  
  % get user tunes
  dtunes = dbload(self.fetch_file_name('read','scenario_tunes', sc));  
  dtunes = cmop.preprocess_database(dtunes, opt.rs_maturities);
  
  % get filtered database
  if isfield(opt, 'hist_use') && isfield(opt.hist_use, sc)
    sch = opt.hist_use.(sc);
  else
    sch = sc;
  end
  dhist = dbload(self.fetch_file_name('read', 'scenario_history_mean', sch));
  [dhisto, fchar] = self.create_other_freq_database(dhist, sc, opt);
  
  % output databases
  ds = {dhist};
  dos = {dhisto};

  % loop over the srng
  for ii = 1:length(srng)
    sdate = srng(ii);
    disp(['### Rolling forecast for ' sc ' starting ' dat2char(sdate)]);
    % clip the last database
    dhistrun = dbclip(ds{ii}, opt.history_range(1):sdate-1);
    % take future shocks besides shocks_ignore
    dshocks = dbbatch(dhist, '$0', 'resize(dhist.$0, sdate:sdate+args.ns-1)',...
                       'nameList', get(m, 'eList') - shocks_ignore, 'fresh', true);
    % create input database
    din = dboverlay(dhistrun, dtunes);
    din = dboverlay(dshocks, din);
    %- overlay it with steady state database for lags
    dlags = sstatedb(m, sdate);
    dlags = dbclip(dlags, sdate-1+get(m,'maxlag'):sdate-1);
    din = dboverlay(dlags, din);

    % set homotopy steps
    siropt = sirius_options.default();
    siropt.set_option('homotopy_steps',hss(ii));
    siropt.apply();

    % do the forecast
    [d, r] = cmop.combined_forecast(mm, sdate:fend, din, ...
                                    struct(), ...
                                    {});
    d = cmop.postprocess_database(d, opt.rs_maturities);
    [do, fchar] = self.create_other_freq_database(d, sc, opt);
    % and save to ds and dos
    ds{end+1} = d;
    dos{end+1} = do; 
  end
  
  % get the forecast report definition
  f = self.fetch_file_name('run','define_forecast_report');
  [p,n,e] = fileparts(f);
  p = pathincr(p);
  clear(n);
  if ~isstruct(opt.model_linear), swt = opt.model_switches; ...
  else swt = opt.model_switches.(sc); end
  rd = eval([n '(swt);']);
  %- remove all graphs from the forecast report
  rdd = {};
  for i=1:length(rd)
    if isfield(rd{i}, 'table'), rdd{end+1} = rd{i};; end
  end

  % one forecast report
  reptitle = ['Rolling forecast report for ' sc ...
              ': ' dat2char(srng(1)) ' -- ' dat2char(srng(end))];
  x = report.new(reptitle);
  for ii = 1:length(srng)
    sdate=srng(ii);
    x = tex(x, '', ['\hbox{}\vskip3cm \hbox to \hsize{\hss\Large Forecast starting ' ...
                    'in ' dat2char(sdate) '\hss}']);
    x = pagebreak(x);
    % prepare table ranges
    theopt = opt;
    theopt.report_table_range = sdate-4:min(sdate+12,fend);
    theopt.report_table_range2 = yy(dat2ypf(theopt.report_table_range(1))):...
        yy(dat2ypf(theopt.report_table_range(end)));
    % prepare names
    name = ['start' dat2char(sdate)];
    nameprev = ['start' dat2char(sdate-1)];
    if sdate == srng(1) || strcmp(args.compareto,'hist'), nameprev = 'history'; end
    icmp = ii; if strcmp(args.compareto,'hist'), icmp = 1; end
    x = cmop.process_forecast_report(x, m, {ds{ii+1}, ds{icmp}},...
                                     {dos{ii+1}, dos{icmp}}, {}, struct(), ...
                                     {name, nameprev}, rdd, theopt);
  end
  filename = [self.dir filesep 'reports' filesep self.name '_' ...
              sc '_rolling_forecasts_' dat2char(srng(1)) '_' dat2char(srng(end)) ...
              '.pdf'];
  x.publish(filename, 'display',false,'package',{'relsize'});
  pdflist = {filename};
  
  
  % save the databases
  csvlist = {};
  fname = [self.dir filesep 'results' filesep self.name '_' ...
             sc '_rolling_forecasts_history.csv'];
  dbsave(ds{1}, fname);
  csvlist{end+1} = fname;
  for ii=1:length(srng)
    fname = [self.dir filesep 'results' filesep self.name '_' ...
             sc '_rolling_forecasts_' dat2char(srng(ii)) '.csv'];
    dbsave(ds{ii+1}, fname);
    csvlist{end+1} = fname;
  end
end
