% abstract class for dynamic model information

classdef gimi < gmi
    
  properties 
    % IRIS model object
    model = model();
    % extra substitutions to be done when processing the equations
    eq_subst = struct('pattern',{},'subst',{});
  end

  methods     
    % contructor
    function self = gimi(m,varargin)
        self.model = m;
        % process opt
        opt = struct(varargin{:});
        if isfield(opt,'eq_subst')
          self.eq_subst = opt.eq_subst;
        end
    end
    
    % get model name
    % 
    % r = self.model_name();
    % 
    %  self   object of this gmi class
    %  r      returned string
    function r = model_name(self)
        r = comment(self.model);
        if isempty(r)
           r = 'Model'; 
        end
    end
    
    % return model symbols, labels, units. etc
    %
    % [endo, exo, params] = self.model_symbols();
    %
    %  self   object of this gmi class
    %  endo   struct array with fields:
    %            name*        string of the variable name
    %            is_additive* true/false
    %            label        string describing the variable
    %            unit         string describing the unit
    %            latexAlias   string for LaTeX rendering of the variable
    %                         name
    %  exo    struct array with fields:
    %            name*        string of the variable name
    %            label        string describing the variable
    %            unit         string describing the unit
    %            latexAlias   string for LaTeX rendering of the variable
    %                         name
    %  params struct array with fields:
    %            name*        string of the variable name
    %            label        string describing the variable
    %            unit         string describing the unit
    %            latexAlias   string for LaTeX rendering of the variable
    %                         name
    %
    % field* means that the field is mandatory
    function [endo, exo, params] = model_symbols(self)
        loglist = get(self.model,'log');
        xlist = get(self.model,'xList');
        xlabs = get(self.model,'xDescript');
        endo(length(xlist)) = struct;
        for i = 1 : numel(xlist)
            endo(i).name = xlist{i};
            if loglist.(xlist{i})
                endo(i).is_additive = false;
            else
                endo(i).is_additive = true;
            end
            endo(i).label = xlabs{i};
            endo(i).unit = '';
            endo(i).latexAlias = '';
        end
        elist = get(self.model,'eList');
        elabs = get(self.model,'eDescript');
        exo(length(elist)) = struct;
        for i = 1 : numel(elist)
            exo(i).name = elist{i};
            exo(i).label = elabs{i};
            exo(i).unit = '';
            exo(i).latexAlias = '';
        end        
        plist = get(self.model,'pList');
        plabs = get(self.model,'pDescript');
        params(length(plist)) = struct;
        for i = 1 : numel(plist)
            params(i).name = plist{i};
            params(i).label = plabs{i};
            params(i).unit = '';
            params(i).latexAlias = '';
        end
    end

    % return model equations
    %
    %   self   object of this gmi class
    %   r      returned cell array of structs with fields
    %             lhs   string fro the left hand side
    %             rhs   string fro the right hand side
    function r = model_equations(self)
        eqlist = get(self.model,'xEqtn'); 
        eqlabs = get(self.model,'xLabels'); 
        r = cell(length(eqlist));
        for i = 1 : numel(eqlist)
            tmp_eqtn = regexprep(eqlist{i},';','');
            tmp_eqtn = regexprep(tmp_eqtn,'#','');
            for iix = 1 : length(self.eq_subst)
                tmp_eqtn = regexprep(tmp_eqtn,self.eq_subst(iix).pattern,...
                  self.eq_subst(iix).subst);
            end
            [lhs,rhs] = strtok(tmp_eqtn,'=');
            r{i} = struct;
            r{i}.lhs = lhs;
            r{i}.rhs = rhs(2:end);
        end
    end
    
    % return values of the variables and parameters
    %
    %   self   object of this gmi class
    %   levels struct with fields endo variables and values steady state
    %          values
    %   trends struct with fields endo variables and values the trends
    %   params struct with fields parameters and values the param values
    function [levels, trends, params] = model_values(self)
        ssl = get(self.model,'sstateLevel');
        ssg = get(self.model,'sstateGrowth');
        pms = get(self.model,'params');
        xlist = get(self.model,'xlist');
        elist = get(self.model,'elist');
        plist = get(self.model,'plist');
        vlist = [xlist, elist];
        levels = struct;
        trends = struct;
        params = struct;
        for i = 1 : numel(vlist)
            levels.(vlist{i}) = ssl.(vlist{i});
            trends.(vlist{i}) = ssg.(vlist{i});
        end
        for i = 1 : numel(plist)
           params.(plist{i}) = pms.(plist{i}); 
        end
    end
    
    % return autoexogenised variable and shock pairs
    %
    %   self   object of this gmi class
    %   autoex struct with fields endo variables and values shock names
    function [autoex] = model_autoexogenise(self)
        autoex = autoexogenise(self.model);
    end
  end
end