% Retrieve optimal period-on-period growth rates so that they replicate
% given year-on-year growth rates
%
% d     input time-serios of growth rates in log-differences
% x     output time-series of period-on-period growth rates in log
%       differences
%
% Optimization chooses f-1 degrees of freedom, where f is a frequency, in
% the initial part of x so that it matches year-on-year and minimizes
% changes of period-on-period growth rate, that is x(t)-x(t-1)

function x = calculate_growth_from_overlapping_growths(d)

  % get frequency
  f = get(d, 'freq');
 
  % construct constraints: period-on-period must sum up to year-on-year
  rng = get(d, 'range');
  N = length(rng);
  B = zeros(N, N+f-1);
  for i = 1:f+1-1
    ii = [1:N];
    jj = [1:N]+i-1;
    kk = sub2ind(size(B), ii, jj);
    B(kk) = 1/f;
  end
  b = d(rng);
  
  % construct optimization objective: minimize first difference of x
  A = zeros(N+f-2, N+f-1);
  ii = [1:N+f-2];
  jj = [1:N+f-2];
  kk1 = sub2ind(size(A), ii, jj);
  kk2 = sub2ind(size(A), ii, jj+1);
  A(kk1) = 1;
  A(kk2) = -1;
  a = zeros(N+f-2,1);
  
  % calculate with the constrained lsq
  x = constrained_lsq(A, a, B, b);
  x = tseries(rng(1)-f+1:rng(end), x);
  
end
