% this function adds reer and competitve reer data to the database dbout for
% given countries and the given counter-parties

% countries has the form:
% countries = {'zge','zge_gel_usd','zge_cpi','Georgia',...
%              'ztn','ztn_tnd_usd','ztn_cpi','Tunisia'};
% where the first is a country code, second is the LCY/USD exchange rate,
% third is CPI index level in LCY, and third is a name of the country as
% appears in IMF DOTS database transformed by
% codes/matlab/general/trade_matrix/process_dots.m:convert_name

% cparties has the form:
% cparties = {'us','us_usd_usd','us_cpi','United_States',...
%             'ez','ez_eur_usd','ez_cpi','Eurozone',...
%             'ru','ru_rub_usd','ru_cpi','Russian_Federation'};


function dbout = construct_reer(db, trade_matrix_func, countries, cparties)
  
  dbout = dbempty();
  
  nc = length(countries)/4; % number of countries
  np = length(cparties)/4; % number of counter-parties
  n = nc+np;
  cntys = {countries{:}, cparties{:}};
  listall = cntys(4:4:end);
  
  % get the trade matrix and do transformations
  [data,labels] = eval(trade_matrix_func);
  [data,labels] = replace_eurozone(data, labels);
  [data,labels] = replace_china(data, labels);
  [data,labels] = restrict(data, labels, listall);
  
  % get mindate and maxdate as a common range of cparties data
  mindate = -Inf;
  maxdate = Inf;
  for i=1:np
    for j=2:3
      sname = cparties{(i-1)*4+j};
      if get(db.(sname),'start') > mindate
        mindate = get(db.(sname),'start');
      end
      if get(db.(sname),'end') < maxdate
        maxdate = get(db.(sname),'end');
      end
    end
  end
  
  % calculate bilateral rers: bilrer{i,j} is i-currency/j-currency
  bilrer = cell(n,n);
  for i=1:n
    for j=1:i-1
      s_i_usd = db.(cntys{(i-1)*4+2});
      s_j_usd = db.(cntys{(j-1)*4+2});
      p_i = db.(cntys{(i-1)*4+3});
      p_j = db.(cntys{(j-1)*4+3});
      bilrer{i,j} = s_i_usd*p_j/s_j_usd/p_i;
      % prolong to mindate with hpf
      if get(bilrer{i,j},'start') > mindate
        ds = get(bilrer{i,j},'start');
        lx = log(bilrer{i,j});
        xbar = exp(hpf(lx, mindate:maxdate));
        bilrer{i,j}(mindate:ds-1) = xbar(mindate:ds-1);
      end
      bilrer{j,i} = 1/bilrer{i,j};
    end
  end
  
  % calculate weights: normalize rows of trade matrix
  % w(i,j) is i-th country exports to j-th country
  w = diag(sum(data,2))\data;
  
  % calculate reers
  for i=1:nc
    cc = cntys{(i-1)*4+1};
    reer = tseries(mindate:maxdate,1);
    for j=1:n
      if j ~= i
        reer = reer*bilrer{i,j}^w(i,j);
      end
    end
    dbout.([cc '_reer']) = reer;
  end
  
  % calculate competitive reers
  for i=1:nc
    cc = cntys{(i-1)*4+1};
    creer = tseries(mindate:maxdate,1);
    for j=1:n
      if j ~= i
        reerj = bilrer{i,j};
        for k=1:n
          if k ~= i && k ~= j
            reerj = reerj/(bilrer{k,j}^w(k,j));
          end
        end
        creer = creer*reerj^w(i,j);
      end
    end
    dbout.([cc '_creer']) = creer;
  end

  % set range of reers according to the country's cpi and exchange rate
  for i=1:nc
    cc = cntys{(i-1)*4+1};
    fxname = cntys{(i-1)*4+2};
    cpiname = cntys{(i-1)*4+3};
    dsfx = get(db.(fxname),'start');
    dscpi = get(db.(cpiname),'start');
    ds = max([dsfx, dscpi, mindate]);
    dbout.([cc '_reer']) = resize(dbout.([cc '_reer']), ds:maxdate);
    dbout.([cc '_creer']) = resize(dbout.([cc '_creer']), ds:maxdate);
  end
end
