function iris2tex2(model, model_description, output_filename, varargin)

% function: iris2tex2(...)
% Author:   O. Auda
% Contact:  olda@auda.cz
% Version:  1.2
% Date:     2011-01-30

% Usage:
% iris2tex2(model, model_description, output_filename, varargin)
% where
%   - model is iris model, for example 
%     model('/ogi/modules/models/kh/kh.mod', 'linear', true, 'assign',
%     p);
%     where p is for example from /ogi/modules/models/kh/kh_setparam.m
%   - model_description is ogi handled description xml, for example 
%     model_description('/ogi/modules/models/kh/kh_description.xml', '/ogi/modules/codes/xml/model_description.dtd');
%   - output_filename is filename of latex output, for example 
%     'iris2tex_output.tex'
%   - varargin (optional): 'modelcode', 'eqallbreak'
% 
% The resulting latex file should be included together with general macros:
%   \input{iris2tex_output.tex}
%   \input{iris2tex2general.tex}
% 
% New available tex macros for use inside document:
% 
%   \eqgivenlabel (for example \eqdemandside if iris label contain string 
%                 [demandside])
%   \eqnr{1}, \eqnr{2}, \eqnr{3}...
%
% These equations can be referenced by their label like this 
%   \ref{kh: eqdemandside}
% or like this in case label doesn't exist:
%   \ref{kh: eq14}
%
% New available tex macros for use in appendix:
%
%   \varendogenous
%   \varendogenousloglinear
%   \varexogenous
%   \varsteadystate
%   \varparameters
%   \varadditional
%   \eqall
%
% More details:
%
% Insert labelled equation
% - if named by [demandside] in iris comment then \eqdemandside
% - in any case by order (it will be changed when model changed) \eqnr{14}
%
% List of endogenous variables
% - \varendogenous
%
% List of endogenous log-linear variables
% - \varendogenousloglinear
%
% List of exogenous variables
% - \varexogenous
%
% List of parameters
% - \varparameters
% - ignored section of for example kh_setparam.m: "STD ERRORS OF SHOCKS"
%
% List of all equations
% - \eqall
% - section is defined by iris comment (part in brackets [foo] is ignored)
%
% ToDo:
% - TeX improvement
%   + var list description add unit (conditionally)
%   + var list table cols width
%   + var list width parametrization
%   + var list for log linear maths in right col
%   + var list for parameters: NaN, empty name etc.
%   + * in equations
%   + equations - allow more lines + auto split: breqn
%   + mathpozo (suggested by breqn) conflicts with \Delta, therefore
%     disabled
%   - set breqn \bop@incr to smaller penalty than default 4444 (later)
% + ogi integration
% + split 'Steady-State Parameters' table to two tables 
%   'Steady-State Parameters' and 'Transition Parameters'
% + tex notation of expectations
%     x{n} -> E_t[LX_{t+n}]
%     x{+n} -> E_t[LX_{t+n}]
%   where
%     x  .. matlab variable name
%     LX .. its LaTeX notation
%     n  .. any natural number excluding 0
% + for equations inside text part use same label (A3.c) as in appendix
%   part
% + fix overstrip in tex for variables (leads to "\Deltaz" instead of
%   "\Delta z"
% - check on windows miktex 2.8 (Adam)
%

  eq_label_prefix = 'eq'
  %eq_label_prefix = sprintf('%s: eq', '\CtryCode');
  var_list_desc_width = '8cm'
  var_list_param_desc_width = '10cm'
  var_list_addit_width = '11cm'

  %% read optional arguments
  modelcode = '';
  eqallbreak = '';
  eqallbreak_dict = struct();
  skipequations = {};
  for iter=1:2:length(varargin)
    switch lower(varargin{iter})
      case 'modelcode'
        modelcode = varargin{iter+1}
      case 'eqallbreak'
        eqallbreak = varargin{iter+1}
        eqallbreak_dict = get_eqallbreak_dict(eqallbreak);
      case 'skipequations'
        if ~isempty(varargin{iter+1})
          skipequations = regexp(varargin{iter+1},';','split');
        end
      otherwise
        error('Unknown optional argument: %s', varargin{iter});
    end
  end
  
  skipeq = struct();
  for i = 1 : length(skipequations)
    skipeq.(skipequations{i}) = 1;
  end
  
  %if ~strcmp(eqallbreak, '')
  %  eqallbreakdict = struct();
  %  eqallbreakmat = textscan(eqallbreak, '%d,%d,%s', 'delimiter', ';')
  %else
  %  eqallbreakdict = struct();
  %end

  % model input
  equations = get(model, 'xEqtn');
  labels = get(model, 'xLabels');
  sstates = get(model, 'sState');
  mlog = get(model, 'log');

  % tex output
  fout = fopen(output_filename, 'w');
  fprintf(fout, '%s\n\n', '%% iris2tex2');

  % equations
  fprintf(fout, '%s\n\n', '%% equations');
  
  
  eqall = '';
  for i = 1 : length(equations)
    % find label if exist
    label_all = labels{i};
    [section, label] = label_to_tex(label_all);

    % skip equations with label in attributes
%     if isfield(skipeq, label)
%       continue
%     end
    
    % tex doesn't like \eq1 - order_to_texid is really nasty solution  FIXME
    label_i = order_to_texid(i);
    
    % tex label (latex label ref pairs)
    if label
      eq_label = sprintf('%s%s', eq_label_prefix, label);
    else
      eq_label = sprintf('%s%d', eq_label_prefix, i);
    end
    %tex_label = sprintf(' \\label{%s}', eq_label);
    tex_label = eq_label;
    
    % parse matlab to tex 
    [tex_eq, tex_eq_all] = eq_to_tex(equations{i}, model_description, i, ...
                                     tex_label, modelcode, eqallbreak_dict);
    
    % tex output
    fprintf(fout, '%%%% %d %s %s\n', i, label_i, label_all);
    fprintf(fout, '%%%% %s\n', equations{i});
    fprintf(fout, '\\def\\eqint%s{%s}\n', label_i, tex_eq);    
    fprintf(fout, '\\def\\eqallint%s{%s}\n', label_i, tex_eq_all);
    if label
      fprintf(fout, '\\def\\eq%s{\\eqint%s}\n', label, label_i);
      fprintf(fout, '\\def\\eqall%s{\\eqallint%s}\n', label, label_i);
    end
    fprintf(fout, '\n');
    
    % tex output \def\eqall
    if and(i == 1, not(section))
      section = 'No section label'
      block_count = 0;
    end
    if section
      if i == 1
        block_end = '';
      else
        %block_end = sprintf('\\end{align}\n\n');
        block_end = sprintf('\\end{dgroup}\n\n');
      end
      %eqall = sprintf('%s\n%s\n\\vspace{8 mm}\\textbf{%s:}\n\n\\begin{align}\n', ...
      %                eqall, block_end, section);
      eqall = sprintf('%s\n%s\n\\vspace{8 mm}\\textbf{%s:}\n\n\\begin{dgroup}\n', ...
                      eqall, block_end, section);
      block_count = 1;
    elseif block_count >= 26
      block_end = sprintf('');
      eqall = sprintf('%s\n\\end{dgroup}\n\\begin{dgroup}\n', eqall);
      block_count = 1;
    else
      %eqall = sprintf('%s \\\\\n', eqall);
      eqall = sprintf('%s \n', eqall);
      block_count = block_count + 1;
    end
    eqall = sprintf('%s\\eqallint%s', eqall, label_i);
    if i == numel(equations)
      %block_end = sprintf('\\end{align}\n\n');
      block_end = sprintf('\\end{dgroup}\n\n');
      eqall = sprintf('%s\n%s\n', eqall, block_end);
    end
  end
  
  % tex output \def\eqall
  fprintf(fout, '%s\n\n', '%% equations - all');
  fprintf(fout, '\\def\\eqall{%%\n');
  %for i = 1 : length(equations)
  %  fprintf(fout, '\\eqint%s\n', label_i);
  %end
  fprintf(fout, '%s\n', eqall);
  fprintf(fout, '}\n\n');

  % tex matlab vars
  varendo = '';
  varendologlin = '';
  varexo = '';
  varss = '';
  varpar = '';  
  varadd = '';
  
  count_endo = 0;
  count_endologlin = 0;
  count_exo = 0;
  count_ss = 0;
  count_par = 0;  
  count_add = 0;
  
  
  model_desc_names=fieldnames(model_description);
  % 1 in sstates , 2 in mlog, 3 report - 1 , 4 report-empty, 5 report - 0 , 
  var_location_info=zeros(1,length(model_description));

  for i = 1:numel(model_desc_names)  
    mdtok_name=model_desc_names{i};
    if isfield(mlog, mdtok_name)
      var_location_info(i)=2;
    elseif isfield(sstates, mdtok_name)
      var_location_info(i)=1;
    else
      mdtok=model_description.(mdtok_name);
      if isfield(mdtok,'report')
        display(mdtok_name);
        report_val=regexprep(mdtok.report, '\s', '');
        if ~length(report_val)
          var_location_info(i)=4;
        elseif strcmp(report_val,'1')
          var_location_info(i)=3;
        elseif strcmp(report_val,'0')
          var_location_info(i)=5;
        else
          % var_location_info(i)=0
        end
      else
        % var_location_info(i)=0
      end
    end    
  end
  
  % fill varadd array
  for i = 1:length(var_location_info)
    stat=var_location_info(i);
    if stat==3 % 3 means report
      mdtok=model_description.(model_desc_names{i});
      texlat = get_texlatin_repr(model_description,model_desc_names{i});
      if ~isfield(mdtok, 'long')
        warning(['long element missing in description of ts ' model_desc_names{i} ', will be set empty']);
        mdtok.long = '';
      end
      unit = '';
      if isfield(mdtok, 'unit')
        if ~isempty(mdtok.unit)
          unit=sprintf(' (%s)', mdtok.unit);
        end
      end
      desc = sprintf('%s%s', mdtok.long, unit);
      desc = regexprep(desc, '_', '{\\_}');
      if isempty(regexprep(desc, '\s', ''))
        disp(sprintf('Warning: Empty description for variable: %s', model_desc_names{i}));
        desc = '';
      end
      mdtex = sprintf('$%s$ & $%s$ & %s \\\\ \n', mdtok.tex, texlat, desc);
      varadd=sprintf('%s%s',varadd,mdtex);
      count_add = count_add + 1 ; 
    end      
  end    
  
  %loop over all sstates properties, except loglin, fill varendo, varexo
  %etc
  flds = fieldnames(sstates);
  for i = 1:numel(flds) 
    token = flds{i};

    % <check_block> (copy check_block to loglin section when changed)
    mdtoken = model_description.(token);
    if ~isfield(mdtoken, 'nametype')
      ntype = 'unknown';
    else
      ntype = mdtoken.nametype;
    end
    texlat = get_texlatin_repr(model_description,token);
    if ~isfield(mdtoken, 'tex')
      warning(['tex element missing in description of ts ' token ', will be empty']);
      mdtoken.tex = '';
    end
    if ~isfield(mdtoken, 'long')
      warning(['long element missing in description of ts ' token ', will be set empty']);
      mdtoken.long = '';
    end
    if numel(token) >= 4
      if strcmp(token(1:4), 'std_')
        ignore = true;
      elseif strcmp(token(1:4), 'obs_')
        ignore = true;
      else
        ignore = false;
      end
    else
      ignore = false;
    end
    if isfield(mdtoken, 'unit')
      unit = mdtoken.unit;
    else
      unit = '';
    end
    if unit
      unit = sprintf(' (%s)', unit);
    end
    if strcmp(ntype, 'unknown')
      if ignore
        disp(sprintf('Info: Ignored variable: %s', token));
        desc = sprintf('%s%s', mdtoken.long, unit);
        desc = regexprep(desc, '_', '{\\_}');
      else
        disp(sprintf('Warning: Unknown variable (std): %s', token));
        desc = sprintf('%s%s', mdtoken.long, unit);
        desc = regexprep(desc, '_', '{\\_}');
      end
    elseif ignore
      disp(sprintf('Info: Ignored obs_ variable: %s', token));
      desc = sprintf('%s%s', mdtoken.long, unit);
      desc = regexprep(desc, '_', '{\\_}');
    else
      desc = sprintf('%s%s', mdtoken.long, unit);
      desc = regexprep(desc, '_', '{\\_}');
      if not(regexprep(desc, '\s', ''))
        disp(sprintf('Warning: Empty description for variable: %s', token));
        desc = '';
      end
    end
    % </check_block>

    if ~(isfield(mdtoken,'tex'))
       disp(['Missing Tex notation in description for: ',desc]);
    end
    
    if ignore
    elseif strcmp(ntype, 'endogenous')
      isnot_loglin = true;
      if isfield(mlog, token)
        if mlog.(token)
          isnot_loglin = false;
        end
      end
      if isnot_loglin
         mdtex = sprintf('$%s$ & $%s$ & %s \\\\ \n', mdtoken.tex, texlat, desc);        
         varendo = sprintf('%s%s', varendo, mdtex);
         count_endo = count_endo + 1;
        % FIXME
      end
    elseif strcmp(ntype, 'exogenous')
      mdtex = sprintf('$%s$ & $%s$ & %s \\\\ \n', mdtoken.tex, texlat, desc);
      varexo = sprintf('%s%s', varexo, mdtex);
      count_exo = count_exo + 1;
    elseif strcmp(ntype, 'parameter')
      value = sprintf('%g', sstates.(token));
      mdtex = sprintf('$%s$ & $%s$ & %s \\\\ \n', ...
                      mdtoken.tex, value, desc);
      is_ss = false;
      if numel(token) > 3
        if strcmp(token(1:3), 'ss_')
          is_ss = true;
        end
      end
      if is_ss
        varss = sprintf('%s%s', varss, mdtex);
        count_ss = count_ss + 1;
      else
        varpar = sprintf('%s%s', varpar, mdtex);
        count_par = count_par + 1;
      end
    end
  end
  
  
  %loop over loglin , fill varendologlin
  logvarflds = fieldnames(mlog);  
  for i = 1 : numel(logvarflds)
    if mlog.(logvarflds{i})
      token = logvarflds{i};

      % <check_block> (copied, don't change directly here)
    mdtoken = model_description.(token);
    if ~isfield(mdtoken, 'nametype')
      ntype = 'unknown';
    else
      ntype = mdtoken.nametype;
    end
    texlat = get_texlatin_repr(model_description,token);
    if ~isfield(mdtoken, 'tex')
      warning(['tex element missing in description of ts ' token ', will be empty']);
      mdtoken.tex = '';
    end
    if ~isfield(mdtoken, 'long')
      warning(['long element missing in description of ts ' token ', will be set empty']);
      mdtoken.long = '';
    end
    if numel(token) >= 4
      if strcmp(token(1:4), 'std_')
        ignore = true;
      elseif strcmp(token(1:4), 'obs_')
        ignore = true;
      else
        ignore = false;
      end
    else
      ignore = false;
    end
    if isfield(mdtoken, 'unit')
      unit = mdtoken.unit;
    else
      unit = '';
    end
    if unit
      unit = sprintf(' (%s)', unit);
    end
    if strcmp(ntype, 'unknown')
      if ignore
        disp(sprintf('Info: Ignored variable: %s', token));
        desc = sprintf('%s%s', mdtoken.long, unit);
        desc = regexprep(desc, '_', '{\\_}');
      else
        disp(sprintf('Warning: Unknown variable (std): %s', token));
        desc = sprintf('%s%s', mdtoken.long, unit);
        desc = regexprep(desc, '_', '{\\_}');
      end
    elseif ignore
      disp(sprintf('Info: Ignored obs_ variable: %s', token));
      desc = sprintf('%s%s', mdtoken.long, unit);
      desc = regexprep(desc, '_', '{\\_}');
    else
      desc = sprintf('%s%s', mdtoken.long, unit);
      desc = regexprep(desc, '_', '{\\_}');
      if not(regexprep(desc, '\s', ''))
        disp(sprintf('Warning: Empty description for variable: %s', token));
        desc = '';
      end
    end
    % </check_block>
      
      if ~ignore
        mdtex = sprintf('$%s$ & $%s$ & %s \\\\ \n', mdtoken.tex, texlat, desc);
        varendologlin = sprintf('%s%s', varendologlin, mdtex);
        count_endologlin = count_endologlin + 1;
      end
    end
  end
  
  % tex output \def\varendo
  fprintf(fout, '%s\n\n', '%% endogenous variables');
  fprintf(fout, '\\def\\varendogenous{%%\n');
  if count_endo
    var = appendix_var('Transition Variables (linear)', 'tab:Transition_Variables', varendo, 0, var_list_desc_width);
    fprintf(fout, '%s\n', var);
  end
  fprintf(fout, '}\n\n');
  
  % tex output \def\varendologlin
  fprintf(fout, '%s\n\n', '%% endogenous log-lin variables');
  fprintf(fout, '\\def\\varendogenousloglinear{%%\n');
  if count_endologlin
    var = appendix_var('Transition Variables (log-linear)', 'tab:Log-Linear_Variables', varendologlin, 0, var_list_desc_width);
    fprintf(fout, '%s\n', var);
  end
  fprintf(fout, '}\n\n');
  
  % tex output \def\varexo
  fprintf(fout, '%s\n\n', '%% exogenous variables');
  fprintf(fout, '\\def\\varexogenous{%%\n');
  if count_exo
    var = appendix_var('Transition Shocks', 'tab:Transition_Shocks', varexo, 0, var_list_desc_width);
    fprintf(fout, '%s\n', var);
  end
  fprintf(fout, '}\n\n');
  
  % tex output \def\varss
  fprintf(fout, '%s\n\n', '%% steady state parameters');
  fprintf(fout, '\\def\\varsteadystate{%%\n');
  if count_ss
    var = appendix_var('Steady-State Parameters', 'tab:Steady-State_Parameters', varss, 1, var_list_param_desc_width);
    fprintf(fout, '%s\n', var);
  end
  fprintf(fout, '}\n\n');
  
  % tex output \def\varpar
  fprintf(fout, '%s\n\n', '%% parameters');
  fprintf(fout, '\\def\\varparameters{%%\n');
  if count_par
    var = appendix_var('Transition Parameters', 'tab:Transition_Parameters', varpar, 1, var_list_param_desc_width);
    fprintf(fout, '%s\n', var);
  end
  fprintf(fout, '}\n\n');
  
  % tex output \def\varadd
  fprintf(fout, '%s\n\n', '%% additional');
  fprintf(fout, '\\def\\varadditional{%%\n');
  if count_add
    var = appendix_var('Non-model variables', 'tab:Non-model variables', varadd, 2, var_list_desc_width);
    fprintf(fout, '%s\n', var);
  end
  fprintf(fout, '}\n\n');
  
  fclose(fout);
end

% assembles several strings into tex table
function [tex] = appendix_var(title, label, tex_var, mode, longtable_param)
% nasty hack - to be fixed later  FIXME
  if mode==0
    header = sprintf('%s', '\textbf{Variable}&\textbf{Model name}&\textbf{Description}\\ ');
    tabdef = sprintf('llp{%s}', longtable_param);
  elseif mode==1
    header = sprintf('%s', '\textbf{Variable}&\textbf{Value}&\textbf{Description}\\ ');
    tabdef = sprintf('lrp{%s}', longtable_param);
  elseif mode==2
    header = sprintf('%s', '\textbf{Variable}&\textbf{Name}&\textbf{Description}\\ ');
    tabdef = sprintf('llp{%s}', longtable_param);
  end
  tex = sprintf('\n\n%s%s%s%s%s\n%s\n%s\n%s\n%s\n%s%s%s\n%s\n%s\n%s\n%s\n%s\n%s\n\n%s\n\n%s\n%s%s\n%s\n\n', ...
                '\begin{longtable}[c]{', ...
                tabdef, ...
                '}\caption{\footnotesize ', ...
                title, ...
                '} \label{', ...
                label, ...
                '}\\', ...
                '\hline\hline', ...
                header, ...
                '\hline ', ...
                '\endfirsthead ', ...
                '\caption{\footnotesize ', ...
                title, ...
                ' continued} \\ ', ...
                '\hline ', ...
                header, ...
                '\hline ', ...
                '\endhead ', ...
                '\hline ', ...
                '\multicolumn{2}{r}{continued on next page}\\ ', ...
                '\endfoot \\ ', ...
                '\endlastfoot', ...
                tex_var, ...
                '\hline\hline ', ...
                '\end{longtable} ');
end

% tex can't have labels such \eq1, so it has to be emulated like
%   \eqa ( or \eq23 to \eqbc )
function [label_i] = order_to_texid(i)
% nasty hack - to be fixed later  FIXME
  int_to_str = ['a' 'b' 'c' 'd' 'e' 'f' 'g' 'h' 'i', 'j'];
  str_i = sprintf('%d', i);
  label_i = '';
  for j = 1 : length(str_i)
    label_i = strcat(label_i, int_to_str(str2num(str_i(j))+1));
  end
end

function [section, label] = label_to_tex(label_all)
% nasty hack - to be fixed later  FIXME
  section = regexprep(label_all, '\[.*\]', '');
  label_beg = strfind(label_all, '[');
  if label_beg
    %label_end = strfind(label_all(label_beg+1:length(label_all)), ']');
    label_end = strfind(label_all, ']');
    if label_end
      label = label_all(label_beg+1:label_end-1);
    else
      label = '';
    end
  else
    label = '';
  end
end

function [tex, tex_all] = eq_to_tex(eq, md, nr, tex_label, modelcode, eqallbreak_dict)
% eq example:
% l_y_gap_usd = c1_l_y_gap_usd*l_y_gap_usd{-1} + c2_l_y_gap_usd*e_l_y_gap_usd + c3_l_y_gap_usd*rmci_usd{-1} + c4_l_y_gap_usd*l_y_gap_khr{-1} + c5_l_y_gap_usd*l_y_us_gap{-1} + shock_l_y_gap_usd;
% l_y_gap_usd_{t} =c1_l_y_gap_usd_{t}*l_y_gap_usd_{t-1} +c2_l_y_gap_usd_{t}*e_l_y_gap_usd_{t} +c3_l_y_gap_usd_{t}*rmci_usd_{t-1} +c4_l_y_gap_usd_{t}*l_y_gap_khr_{t-1} +c5_l_y_gap_usd_{t}*l_y_us_gap_{t-1} +shock_l_y_gap_usd_{t}

  eq = regexprep(eq, '[\s;]', '');
  eq = sprintf('%s;', eq);  % we need it for: if strcmp(eq(start+length(token)), '{')
  
  [starts, tokens] = regexp(eq, '(\w+)', 'start', 'tokens');
  index = 1;
  tex = '';
  for i = 1 : length(tokens)
    token = tokens(i);
    token = token{1}{1};
    start = starts(i);
    if index <= start
      if regexp(token, '\d+', 'start', 'once') == 1
        if index < start
          i_tex_pre = eq(index:start-1);
          tex = sprintf('%s%s', tex, i_tex_pre);
          index = start;
        end
        tex = sprintf('%s%s', tex, token);
        index = index + length(token);
      else
        if index < start
          i_tex_pre = eq(index:start-1);
          tex = sprintf('%s%s', tex, i_tex_pre);
          index = start;
        end
        i_token = md.(token).tex;
        i_token = regexprep(i_token, '*', '{\\realstar{}}');
        i_token_stripped = regexprep(i_token, '\s', '');
        %i_token = sprintf('{%s}', i_token);
        if or(strcmp(md.(token).nametype, 'endogenous'), strcmp(md.(token).nametype, 'exogenous'))
          if strcmp(eq(start+length(token)), '{')
            i_index = regexp(eq(start+length(token)+1:length(eq)), '}', 'once');
            i_time = eq(start+length(token)+1:start+length(token)+ ...
                        i_index-1);
            is_plus = true;
            is_zero = false;
            if numel(i_time) >= 1
              if strcmp(i_time(1), '-')
                if numel(i_time) == 1
                  is_zero = true;
                end
                is_plus = false;
              elseif strcmp(i_time(1), '+')
                if numel(i_time) >= 2
                  if strcmp(i_time(2), '0')
                    is_zero = true;
                  else
                    plus = '';
                  end
                else
                  disp(sprintf('Warning: var{+}, var_tex{t} used'));
                  is_zero = true;
                end
              elseif strcmp(i_time(1), '0')
                is_zero = true;
              else
                plus = '+';
              end
            else
              disp(sprintf('Warning: empty var{}, E_t{var_tex{t+1}} used'));
              plus = '+1';
            end
            if is_zero
              disp(sprintf('Info: var{0} or var{0} or var{+} for %s', token));
              i_tex = sprintf('%s_{t}', i_token);
            else
              if is_plus
                i_tex = sprintf('E_t[%s_{t%s%s}]', i_token, plus, i_time);
              else
                i_tex = sprintf('%s_{t%s}', i_token, i_time);
              end
            end
            tex = sprintf('%s%s', tex, i_tex);
            index = start+length(token)+i_index+1;
          else
            tex = sprintf('%s%s_{t}', tex, i_token);
            index = start + length(token);
          end
        elseif strcmp(i_token_stripped, 'log')
          tex = sprintf('%s%s', tex, token);
          index = start + length(token);
        elseif strcmp(md.(token).nametype, 'unknown')
          disp(sprintf('Warning: Unknown variable in equation: %s (eq%d)', token, nr));
          tex = sprintf('%s%s', tex, i_token);
          index = start + length(token);
        elseif i_token
          tex = sprintf('%s%s', tex, i_token);
          index = start + length(token);
        else
          disp(sprintf('Warning: Empty tex for variable: %s (eq%d)', token, nr));
          tex = sprintf('%s%s', tex, token);
          index = start + length(token);
        end
      end
    end
  end
  tex = sprintf('%s%s', tex, eq(index:end));
  %tex = regexprep(tex, '[\s;]', '');
  tex = regexprep(tex, ';', '');
  tex = regexprep(tex, '*', '\\mytimes{}');
  
  eq_pos = strfind(tex, '=');
  if eq_pos
    tex_left = tex(1:eq_pos-1);
    tex_eq = '=';
    tex_right = tex(eq_pos+1:end);
  else
    tex_left = '';
    tex_eq = '';
    tex_right = tex;
  end
  
  %% adding breaks into equations
  tex_wholeeq = [ tex_left tex_eq tex_right ];
  if isfield(eqallbreak_dict, ['eq' num2str(nr)])
    eqn = ['eq' num2str(nr)];
    numofbreaks = numel(eqallbreak_dict.(eqn));
    for i = 1:numofbreaks
      tmp = eqallbreak_dict.(eqn){i};
      op_nr = str2num(tmp.nr);
      op_tp = tmp.op;
      idx = strfind(tex_wholeeq, op_tp);
      if numel(idx) >= op_nr
        tex_wholeeq = [tex_wholeeq(1:idx(op_nr)-1) '\break' tex_wholeeq(idx(op_nr):end)]
      end
    end
  end
  
  %% preparing strings for print into tex
  %tex = sprintf('\n\n\\begin{align}\n%s&%s%s%s\n\\end{align}\n\n', tex_left, tex_eq, tex_right, tex_label);
  %tex = sprintf('\n\n\\begin{dmath}[number={\\ref{%s}}]\n%s%s%s\n\\end{dmath}\n\n', ...
  %              tex_label, tex_left, tex_eq, tex_right);
  %vphantom1 = '\vphantom{$\Delta s^{TSB/USD}_{t-1}$}';
  %vphantom2 = '\vphantom{\Delta s^{TSB/USD}_{t-1}}';
  %vphantom1 = '\vrule height20pt depth8pt\relax ';
  %vphantom2 = '\vrule height20pt depth8pt\relax ';
  vphantom1 = '\vphantom{\vrule height12pt depth3pt}';
  vphantom2 = '\vphantom{\vrule height12pt depth3pt}';
  tex = sprintf('\n\n%s%s%s%s%s\n\\begin{dmath*}[nocenter]\n%s%s\n\\end{dmath*}\n\n', ...
                '\vglue 12pt\relax\rlap{\hskip\hsize\llap{', ...
                vphantom1, ...
                '(\ref{', ...
                tex_label, ...
                '})}}\nobreak\vskip -26.35pt\relax\nobreak', ...
                vphantom2, ...
                tex_wholeeq);
  %tex_all = sprintf('%s&%s%s', tex_left, tex_eq, tex_right);
  tex_all = sprintf('\n\n\\begin{dmath}[breakdepth={2},label={%s}]\n%s\n\\end{dmath}\n\n', ...
                    tex_label, tex_wholeeq);
  
end


% eqallbreak looks like '3,2,+;5,2,+;6,2,+'
%  they are triples - first part means number of equation,
%  second and third after what number of which char put break
function eqallbreak_dict = get_eqallbreak_dict(eqallbreak)
  eqallbreak_dict = struct();
  eqallbreak = [eqallbreak ';'];
  idx = strfind(eqallbreak, ';');
  idx = [0 idx]; % indexes of starts of triples
  for i = 2:numel(idx)
    eqallbreak_one = eqallbreak(idx(i-1)+1:idx(i)-1);
    disp(eqallbreak_one);
    idx2 = strfind(eqallbreak_one, ',');
    if numel(idx2) == 2
      eq_nr = eqallbreak_one(1:idx2(1)-1); % which equation
      op_nr = eqallbreak_one(idx2(1)+1:idx2(2)-1); % order of desired operator
      op_tp = eqallbreak_one(idx2(2)+1:end); % which operator
      tmp = struct('nr', op_nr, 'op', op_tp);
      tmp2 = struct(['eq' eq_nr], tmp);
      if ~isfield(eqallbreak_dict, ['eq' eq_nr])
        eqallbreak_dict.(['eq' eq_nr]) = {};
      end
      eqallbreak_dict.(['eq' eq_nr]){numel(eqallbreak_dict.(['eq' eq_nr]))+1} = tmp ;
    end
  end
end

% get texlatin representation for token
% if available use texlatin field, otherwise token name (stripped for TeX)
function texlat_repr = get_texlatin_repr(mod_descript,tokname)
  if isfield(mod_descript.(tokname), 'texlatin')
    texlat=mod_descript.(tokname).texlatin;
    if ~isempty(texlat)
      texlat_repr=texlat;
    else
      texlat_repr=regexprep(tokname, '\s', '');
      texlat_repr=regexprep(texlat_repr, '_', '{\\_}');
    end
  else
    texlat_repr=regexprep(tokname, '\s', '');
    texlat_repr=regexprep(texlat_repr, '_', '{\\_}');
  end
  %disp(texlat_repr);
end
