% Given the state space with no measurement errors:
% x(t) = T*x(t-1) + B*eta(t), where eta(t)~N(0,Q)
% y(t) = Z(t)*x(t); Z(t) depends on time as observations are irregular
%
% this function returns smoothed mean x(t+k) as a function of x(t), and
% y(t+1) through y(t+k)
%
% Input:
% T     matrix (n,n)
% B     matrix (n,ns)
% Q     matrix (ns,ns) positive semidefinite
% Zs    cell array of Z(t), t=t+1,...,t+k, columns of Z(t) are n, each
%       Z(t) has variable number of rows
%
% Output:
% C     matrix (n,n+nobs), where nobs is a sum of rows of Zs, matrix C
%       multiplies vector [x(t);y(t+1);...,y(t+k)] to get x(t+k)
%
% algebra is based on the notes sent from ondra.kamenik@gmail.com on 
% Jun 12, 2012, subject: "mmm: distirbutions of the latest period for
% irregular observations"
% with the error in the sign of T*x(t) [should be T*x(t) instead of
% -T*x(t)]

% Copyright 2012, OGResearch
%

function C = smoother_mean(T, B, Q, Zs)
  
  % dimensions
  n = size(T,1);
  ns = size(B,2);
  k = length(Zs);
  nobs = 0;
  for i = 1:k
    nobs = nobs + size(Zs{i},1);
  end
  % factor Q = Qhat*Qhat^T
  [u,s] = svd(Q);
  Qhat = u*sqrt(s);
  
  % matrix Bhat = B*Qhat;
  Bhat = B*Qhat;
  
  % state space for period t+1 through t+k in block form:
  % E*[x(t+1);..x(t+k)] = a + F*whitenoise(t)
  E = zeros(k*n+nobs, k*n);
  F = zeros(k*n+nobs, k*ns);
  nacc = 0;
  for i=0:k-1
    E(i*n+1:(i+1)*n, i*n+1:(i+1)*n) = eye(n,n);
    F(i*n+1:(i+1)*n, i*ns+1:(i+1)*ns) = Bhat;
    if i > 0
      E(i*n+1:(i+1)*n, (i-1)*n+1:i*n) = -T;
    end
    nobsi = size(Zs{i+1});
    E(k*n+nacc+1:k*n+nacc+nobsi, i*n+1:(i+1)*n) = Zs{i+1};
    nacc = nacc + nobsi;
  end
  
  % QR decomp of E
  [QQ,RR] = qr(E);
  R22 = RR((k-1)*n+1:k*n,(k-1)*n+1:k*n);
  
  % condense vector a and the working of QQ^T on a
  AA = [QQ(1:n,:)'*T, QQ(n*k+1:end,:)'];
  AA2 = AA((k-1)*n+1:k*n, :);
  AA3 = AA(k*n+1:end, :);
  
  % RQ decomposition of QQ^T*F
  [SS,PP] = rq(QQ'*F);
  S22 = SS((k-1)*n+1:k*n, k*ns-nobs+1:k*ns);
  S32 = SS(k*n+1:k*n+nobs, k*ns-nobs+1:k*ns);
  
  % and form matrix C
  C = pinv(R22)*(AA2 - S22*pinv(S32)*AA3);
  
end
