% Given the state space with no measurement errors:
% x(t) = T*x(t-1) + B*eta(t), where eta(t)~N(0,Q)
% y(t) = Z(t)*x(t); Z(t) depends on time as observations are irregular
%
% this function return variance covariance matrix of smoothed mean of x(t+k)
% provided that x(t), and y(t+1) through y(t+k) are unconditionally
% distributed
% Input:
% T     matrix (n,n)
% B     matrix (n,ns)
% Q     matrix (ns,ns) positive semidefinite
% Zs    cell array of Z(t), t=t+1,...,t+k, columns of Z(t) are n, each
%       Z(t) has variable number of rows
%
% Output:
% V     variance-covariance matrix of smoother mean of x(t+k), dimension (n,n)
%
% The function currently kills unit roots in the transformation matrix and
% calculates the unconditional distribution using doubling algorithm. It
% needs to be improved in these directions:
% 1. separate unit roots eigenspace and stable roots eigenspace and do all
%    the algebra only for stable roots; the results will be interpretted as
%    us uncodnitional variance of smoother_mean of the stable partof the
%    state space
% 2. get rid of doubling algorithm as it is numerically not very stable
%    and use Ondra Kamenik's recursive algorithm

% Copyright 2012, OGResearch
%
function V = unconditional_smoother_mean(T, B, Q, Zs)
  
  % dimensions
  n = size(T,1);
  ns = size(B,2);
  k = length(Zs);
  nobs = 0;
  for i = 1:k
    nobs = nobs + size(Zs{i},1);
  end
  % factor Q = Qhat*Qhat^T
  [u,s] = svd(Q);
  Qhat = u*sqrt(s);
  
  % matrix Bhat = B*Qhat;
  Bhat = B*Qhat;

  % write yy=[x(t);y(t+1);...y(t+k)] as a function of x(t) and a vector
  % of white innovations: yy=AA*x(t)+BB*wn
  AA = zeros(n+nobs,n);
  BB = zeros(n+nobs,k*ns);
  AA(1:n,1:n) = eye(n,n);
  ioff = n;
  TBrun = eye(n,n);
  for i=1:k
    TBrun = [T*TBrun, Bhat];
    ZiTBrun = Zs{i}*TBrun;
    nobsi = size(Zs{i},1);
    AA(ioff+1:ioff+nobsi,1:n) = ZiTBrun(:,1:n);
    BB(ioff+1:ioff+nobsi,1:i*ns) = ZiTBrun(:,n+1:end);
    ioff = ioff + nobsi;
  end
  
  % kill unit roots in T
  tolkill = 1e-5;
  maxeig = 1-tolkill;
  [U,Tr] = schur(T,'real');
  i = 1;
  while i <= n
    if i < n && abs(Tr(i+1,i)) > 1e-12
      % complex eigenvalue
      abseig = sqrt(Tr(i,i)*Tr(i+1,i+1)-Tr(i+1,i)*Tr(i,i+1));
      if abseig > maxeig
        mult = sqrt(maxeig/abseig);
        Tr(i:i+1,:) = mult*Tr(i:i+1,:);
        Tr(:,i:i+1) = mult*Tr(:,i:i+1);
      end
      i = i+2;
    else
      % real eigenvalue
      abseig = abs(Tr(i,i));
      if abseig > maxeig
        mult = sqrt(maxeig/abseig);
        Tr(i,:) = mult*Tr(i,:);
        Tr(:,i) = mult*Tr(:,i);
      end
      i = i+1;
    end
  end
  Tstable = U*Tr*U';
  disp(['unconditional_smoother_mean: largest eigenvalue of T is ' ...
        sprintf('%8.7g',max(abs(eig(Tstable))))]);
  
  % calculate unconditional vcov of x(t) by doubling algorithm
  tol = 1e-12;
  converged = false;
  Vx = Bhat*Bhat';
  Tpow = Tstable;
  while ~converged
    Vincr = Tpow*Vx*Tpow';
    Vx = Vx + Vincr;
    Tpow = Tpow*Tpow;
    converged = norm(Vincr, 'fro') < tol;
  end
  err = Vx-Tstable*Vx*Tstable'-Bhat*Bhat';
  scale = diag(Vx); scale(find(abs(scale)<1e-10))=1;
  disp(['unconditional_smoother_mean: largest relative error in unconditional ' ...
        'vcov is ' num2str(max(max(abs(err/diag(scale)))))]);
  
  % calculate unconditional distribution of yy=AA*x(t)+BB*wn
  Vyy = AA*Vx*AA' + BB*BB';
  
  % calculate deterministic function x(t+k)=C*yy of the smoother mean
  C = smoother_mean(T, B, Q, Zs);
  
  % return the vcov of x(t+k)
  V = C*Vyy*C';
  
end
